@@ -419,18 +419,29 @@ Please check that all strings which were not recognizable to the compiler
            (let ((s (string symbol))) (and (search "THREAD-" s) (search "-SLOT" s)))
            (search "-OFFSET" (string symbol))
            (search "-TN" (string symbol))))
+      (#.(find-package "SB-ALIEN")
+       (or (eq accessibility :external) (eq symbol 'sb-alien::alien-callback-p)))
       ((#.(find-package "SB-C")
         #.(find-package "SB-ASSEM")
         #.(find-package "SB-DISASSEM")
         #.(find-package "SB-IMPL")
+        #.(find-package "SB-FORMAT")
         #.(find-package "SB-UNIX")
         #.(find-package "SB-PCL")
         #.(find-package "SB-MOP")
         #.(find-package "SB-PRETTY")
+        #.(find-package "SB-REGALLOC")
+        #.(find-package "SB-SYS")
+        #.(find-package "SB-UNICODE")
         #.(find-package "SB-BROTHERTREE")
         #.(find-package "SB-KERNEL"))
        ;; Assume all and only external symbols must be retained
        (eq accessibility :external))
+      (#.(find-package "SB-LOOP")
+       (or (eq accessibility :external)
+           ;; Retain some internals to keep CLSQL working.
+           (member symbol '(sb-loop::*loop-epilogue*
+                            sb-loop::add-loop-path))))
       (#.(find-package "SB-THREAD")
        (or (eq accessibility :external)
            ;; for some reason a recent change caused the tree-shaker to drop MAKE-SPINLOCK
