@@ -107,7 +107,7 @@ distinct from the global value. Can also be SETF."
 
 (defun symbol-function (symbol)
   "Return SYMBOL's current function definition. Settable with SETF."
-  (%coerce-name-to-fun symbol symbol-fdefn))
+  (%coerce-name-to-fun symbol (symbol-fdefn symbol)))
 
 ;; I think there are two bugs here.
 ;; Per CLHS "SETF may be used with symbol-function to replace a global
@@ -144,9 +144,15 @@ distinct from the global value. Can also be SETF."
 ;;; the stub automatically somehow.
 ;;; Furthermore it's annoying that you can't name the CAS function (CAS fn).
 #-compare-and-swap-vops
+(progn
 (defun cas-symbol-%info (symbol old new)
-  (setf (symbol-%info symbol) new)
+  (%primitive sb-vm::set-slot symbol new
+              '(setf symbol-%info) sb-vm:symbol-info-slot sb-vm:other-pointer-lowtag)
   old)
+(defun sb-vm::cas-symbol-fdefn (symbol old new)
+  (%primitive sb-vm::set-slot symbol new
+              '(setf symbol-fdefn) sb-vm:symbol-fdefn-slot sb-vm:other-pointer-lowtag)
+  old))
 
 ;;; Accessors for the dual-purpose info/plist slot
 
@@ -198,9 +204,11 @@ distinct from the global value. Can also be SETF."
 (defun symbol-plist (symbol)
   "Return SYMBOL's property list."
   (let ((list (symbol-%info symbol)))
+    ;; The compiler can't possibly know that the CAR of LIST
+    ;; is also a list (if LIST is a LIST), so force it with a TRULY-THE.
     ;; See the comments above UPDATE-SYMBOL-INFO for a
     ;; reminder as to why this logic is right.
-    (if (%instancep list) nil (car list))))
+    (if (%instancep list) nil (truly-the list (car list)))))
 
 (declaim (ftype (sfunction (symbol t) cons) %ensure-plist-holder)
          (inline %ensure-plist-holder))
@@ -283,14 +291,46 @@ distinct from the global value. Can also be SETF."
   "Return SYMBOL's name as a string."
   (symbol-name symbol))
 
+(define-symbol-macro *id->package*
+    (truly-the simple-vector
+     (sap-ref-lispobj (foreign-symbol-sap "lisp_package_vector" t) 0)))
+(export '*id->package*)
+
 (defun sb-xc:symbol-package (symbol)
-  "Return the package SYMBOL was interned in, or NIL if none."
-  (sb-xc:symbol-package symbol))
+  "Return SYMBOL's home package, or NIL if none."
+  (let ((id (symbol-package-id symbol)))
+    (truly-the (or null package)
+               (if (= id +package-id-overflow+)
+                   (values (info :symbol :package symbol))
+                   (aref *id->package* id)))))
 
 (defun %set-symbol-package (symbol package)
   (declare (type symbol symbol))
-  (%set-symbol-package symbol package))
+  (let* ((new-id (cond ((not package) +package-id-none+)
+                       ((package-id package))
+                       (t +package-id-overflow+)))
+         (old-id (symbol-package-id symbol))
+         (name (symbol-name symbol)))
+    (with-pinned-objects (name)
+      (let ((name-bits (logior (ash new-id (- sb-vm:n-word-bits package-id-bits))
+                               (get-lisp-obj-address name))))
+        (declare (ignorable name-bits))
+        (when (= new-id +package-id-overflow+) ; put the package in the dbinfo
+          (setf (info :symbol :package symbol) package))
+        #-compact-symbol (set-symbol-package-id symbol new-id)
+        #+compact-symbol
+        (with-pinned-objects (symbol)
+          (setf (sap-ref-word (int-sap (get-lisp-obj-address symbol))
+                              (- (ash sb-vm:symbol-name-slot sb-vm:word-shift)
+                                 sb-vm:other-pointer-lowtag))
+                name-bits))))
+    ;; CLEAR-INFO is inefficient, so try not to call it.
+    (when (and (= old-id +package-id-overflow+) (/= new-id +package-id-overflow+))
+      (clear-info :symbol :package symbol))
+    package))
 
+;;; MAKE-SYMBOL is the external API, %MAKE-SYMBOL is the internal function receiving
+;;; a known simple-string, and %%MAKE-SYMBOL is the primitive constructor.
 (defun make-symbol (string)
   "Make and return a new symbol with the STRING as its print name."
   (declare (type string string))
@@ -317,19 +357,26 @@ distinct from the global value. Can also be SETF."
 ;;; It's kinda useless to do that, though not technically forbidden.
 ;;; (It can produce a not-necessarily-self-evaluating keyword)
 
-#+immobile-space
 (defun %make-symbol (kind name)
   (declare (ignorable kind) (type simple-string name))
-  (logior-array-flags name sb-vm:+vector-shareable+) ; Set "logically read-only" bit
-  (if #-immobile-symbols
-      (or (eql kind 1) ; keyword
-          (and (eql kind 2) ; random interned symbol
-               (plusp (length name))
-               (char= (char name 0) #\*)
-               (char= (char name (1- (length name))) #\*)))
-      #+immobile-symbols t ; always place them there
-      (truly-the (values symbol) (sb-vm::make-immobile-symbol name))
-      (sb-vm::%%make-symbol name)))
+  ;; Avoid writing to the string header if it's already flagged as readonly, or off-heap.
+  (when (and (not (logtest (ash sb-vm:+vector-shareable+ 8) (get-header-data name)))
+             (sb-kernel::dynamic-space-obj-p name))
+    (logior-array-flags name sb-vm:+vector-shareable+)) ; Set "logically read-only" bit
+  (let ((symbol
+         (truly-the symbol
+          #+immobile-symbols (sb-vm::make-immobile-symbol name)
+          #-immobile-space (sb-vm::%%make-symbol name)
+          #+(and immobile-space (not immobile-symbols))
+          (if (or (eql kind 1) ; keyword
+                  (and (eql kind 2) ; random interned symbol
+                       (plusp (length name))
+                       (char= (char name 0) #\*)
+                       (char= (char name (1- (length name))) #\*)))
+              (sb-vm::make-immobile-symbol name)
+              (sb-vm::%%make-symbol name)))))
+    (%set-symbol-package symbol nil)
+    symbol))
 
 (defun get (symbol indicator &optional (default nil))
   "Look on the property list of SYMBOL for the specified INDICATOR. If this
@@ -438,8 +485,7 @@ distinct from the global value. Can also be SETF."
 
 (defun keywordp (object)
   "Return true if Object is a symbol in the \"KEYWORD\" package."
-  (and (symbolp object)
-       (eq (sb-xc:symbol-package object) *keyword-package*)))
+  (keywordp object)) ; transformed
 
 ;;;; GENSYM and friends
 
