@@ -298,7 +298,7 @@
 ;;; *around* the get and put operations. It makes no sense to lock the table around
 ;;; individual operations, hence the unsynchronized table.
 
-(define-load-time-global *pathnames*
+(defun make-pathname-intern-table ()
     (let ((h (%make-hash-table (logior (pack-ht-flags-weakness +ht-weak-value+)
                                        (pack-ht-flags-kind 3)
                                        hash-table-userfun-flag)
@@ -310,6 +310,8 @@
                                $1.0)))
       (install-hash-table-lock h)
       h))
+(define-load-time-global *pathnames* (make-pathname-intern-table))
+
 ;;; A pathname is logical if the host component is a logical host.
 ;;; This constructor is used to make an instance of the correct type
 ;;; from parsed arguments.
@@ -472,7 +474,7 @@
         (pattern
          ;; A pattern is only matched by an identical pattern.
          (and (pattern-p wild) (pattern= thing wild)))
-        (integer
+        (bignum
          ;; An integer (version number) is matched by :WILD or the
          ;; same integer. This branch will actually always be NIL as
          ;; long as the version is a fixnum.
@@ -480,7 +482,7 @@
 
 ;;; a predicate for comparing two pathname slot component sub-entries
 (defun compare-component (this that)
-  (or (eql this that)
+  (or (eq this that)
       (typecase this
         (simple-string
          (and (simple-string-p that)
@@ -494,7 +496,9 @@
         (cons
          (and (consp that)
               (compare-component (car this) (car that))
-              (compare-component (cdr this) (cdr that)))))))
+              (compare-component (cdr this) (cdr that))))
+        (bignum
+         (eql this that)))))
 
 ;;;; pathname functions
 
@@ -1732,9 +1736,15 @@ unspecified elements into a completed to-pathname based on the to-wildname."
         (parse-host (logical-chunkify namestr start end)))
       (values host :unspecific (directory) name type version))))
 
-(define-load-time-global *logical-pathname-defaults*
+;;; Return a value suitable, e.g., for preinitializing
+;;; *DEFAULT-PATHNAME-DEFAULTS* before *DEFAULT-PATHNAME-DEFAULTS* is
+;;; initialized (at which time we can't safely call e.g. #'PATHNAME).
+(defun make-trivial-default-logical-pathname ()
   (intern-pathname (make-logical-host :name "") :unspecific nil nil nil nil))
 
+(define-load-time-global *logical-pathname-defaults*
+  (make-trivial-default-logical-pathname))
+
 (defun logical-namestring-p (x)
   (and (stringp x)
        (ignore-errors
