@@ -344,6 +344,30 @@
   (real &optional real) (values integer real)
   (movable foldable flushable recursive))
 
+(defknown unary-truncate (real) (values integer real)
+  (movable foldable flushable no-verify-arg-count))
+
+(defknown unary-truncate-single-float-to-bignum (single-float) (values bignum (eql $0f0))
+   (foldable movable flushable fixed-args))
+(defknown unary-truncate-double-float-to-bignum (double-float)
+    (values #+64-bit bignum #-64-bit integer
+            (and
+             #+(and 64-bit
+                    (not (or riscv ppc64))) ;; they can't survive cold-init
+             (eql $0d0)
+             double-float))
+   (foldable movable flushable fixed-args))
+
+(defknown %unary-truncate-single-float-to-bignum (single-float) bignum
+   (foldable movable flushable fixed-args))
+(defknown %unary-truncate-double-float-to-bignum (double-float) bignum
+   (foldable movable flushable fixed-args))
+
+(defknown sxhash-bignum-double-float (double-float) hash-code
+  (foldable movable flushable fixed-args))
+(defknown sxhash-bignum-single-float (single-float) hash-code
+  (foldable movable flushable fixed-args))
+
 (defknown %multiply-high (word word) word
     (movable foldable flushable))
 
@@ -570,7 +594,8 @@
                     &rest proper-sequence)
   sequence
   (call)
-  :derive-type #'result-type-first-arg)
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
+                                          :preserve-vector-type t))
 
 (defknown #.(loop for info across sb-vm:*specialized-array-element-type-properties*
                   collect
@@ -601,7 +626,8 @@
 (defknown fill ((modifying sequence) t &rest t &key
                 (:start index) (:end sequence-end)) sequence
     ()
-  :derive-type #'result-type-first-arg
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
+                                          :preserve-vector-type t)
   :result-arg 0)
 ;;; Like FILL but with no keyword argument parsing
 (defknown quickfill ((modifying (simple-array * 1)) t) (simple-array * 1) ()
@@ -620,7 +646,8 @@
 (defknown replace ((modifying sequence) proper-sequence &rest t &key (:start1 index)
                    (:end1 sequence-end) (:start2 index) (:end2 sequence-end))
   sequence ()
-  :derive-type #'result-type-first-arg
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
+                                          :preserve-vector-type t)
   :result-arg 0)
 
 (defknown remove
@@ -833,7 +860,8 @@
    &key (:key (function-designator ((nth-arg 0 :sequence t)))))
   sequence
   (call)
-  :derive-type #'result-type-first-arg)
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
+                                          :preserve-vector-type t))
 (defknown sb-impl::stable-sort-list (list function function) list
   (call important-result no-verify-arg-count))
 (defknown sb-impl::sort-vector (vector index index function (or function null))
