@@ -90,6 +90,29 @@
      (- other-pointer-lowtag)
      (* fdefn-raw-addr-slot n-word-bytes)))
 
+;;; Various error-code generating helpers
+(defvar *adjustable-vectors*)
+
+(defmacro with-adjustable-vector ((var) &rest body)
+  `(let ((,var (or (pop *adjustable-vectors*)
+                   (make-array 16
+                               :element-type '(unsigned-byte 8)
+                               :fill-pointer 0
+                               :adjustable t))))
+     ;; Don't declare the length - if it gets adjusted and pushed back
+     ;; onto the freelist, it's anyone's guess whether it was expanded.
+     ;; This code was wrong for >12 years, so nobody must have needed
+     ;; more than 16 elements. Maybe we should make it nonadjustable?
+     (declare (type (vector (unsigned-byte 8)) ,var))
+     (setf (fill-pointer ,var) 0)
+     ;; No UNWIND-PROTECT here - semantics are unaffected by nonlocal exit,
+     ;; and this macro is about speeding up the compiler, not slowing it down.
+     ;; GC will clean up any debris, and since the vector does not point
+     ;; to anything, even an accidental promotion to a higher generation
+     ;; will not cause transitive garbage retention.
+     (prog1 (progn ,@body)
+       (push ,var *adjustable-vectors*))))
+
 ;;;; interfaces to IR2 conversion
 
 ;;; Return a wired TN describing the N'th full call argument passing
@@ -119,6 +142,38 @@
                       (nth n *register-arg-offsets*))
       (make-sc+offset control-stack-sc-number n)))
 
+(defstruct fixed-call-args-state
+  (descriptors -1 :type fixnum)
+  #-c-stack-is-control-stack
+  (non-descriptors -1 :type fixnum)
+  (float -1 :type fixnum))
+
+(declaim (#+sb-xc-host special
+          #-sb-xc-host sb-ext:global
+          *float-regs* *descriptor-args*
+          #-c-stack-is-control-stack *non-descriptor-args*))
+
+(defun fixed-call-arg-location (type state)
+  (let* ((primtype (primitive-type type))
+         (sc (find descriptor-reg-sc-number (sb-c::primitive-type-scs primtype) :test-not #'eql)))
+    (case (primitive-type-name primtype)
+      ((double-float single-float)
+       (make-wired-tn primtype
+                      sc
+                      (elt *float-regs* (incf (fixed-call-args-state-float state)))))
+      ((unsigned-byte-64 signed-byte-64)
+       (make-wired-tn primtype
+                      sc
+                      (elt #-c-stack-is-control-stack *non-descriptor-args*
+                           #+c-stack-is-control-stack *descriptor-args*
+                           (incf (#-c-stack-is-control-stack fixed-call-args-state-non-descriptors
+                                  #+c-stack-is-control-stack fixed-call-args-state-descriptors
+                                  state)))))
+      (t
+       (make-wired-tn primtype
+                      descriptor-reg-sc-number
+                      (elt *descriptor-args* (incf (fixed-call-args-state-descriptors state))))))))
+
 ;;; Make a TN to hold the number-stack frame pointer.  This is allocated
 ;;; once per component, and is component-live.
 (defun make-nfp-tn ()
@@ -149,13 +204,14 @@
 ;;; continuation within a function.
 (defun make-unknown-values-locations (&optional unused-count unused-sp)
   (declare (ignorable unused-count unused-sp))
-  (list (cond #+x86-64
-              ;; needs support from receive-unknown-values, push-values, %more-arg-values
+  (list (cond #+(or arm64 x86-64)
+              ;; needs support from receive-unknown-values, push-values, %more-arg-values, values-list,
+              ;; nlx-entry-multiple
               (unused-sp
                (sb-c::make-unused-tn))
               (t
                (make-stack-pointer-tn)))
-        (cond #+x86-64
+        (cond #+(or arm64 x86-64)
               (unused-count
                (sb-c::make-unused-tn))
               (t
@@ -179,7 +235,6 @@
 ;;; Does the TN definitely hold *any* of the 4 pointer types
 (defun pointer-tn-ref-p (tn-ref)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
-       (tn-ref-type tn-ref)
        (not (types-equal-or-intersect
              (tn-ref-type tn-ref)
              (specifier-type '(or fixnum
@@ -195,7 +250,6 @@
 ;;; Does the TN definitely hold an OTHER pointer
 (defun other-pointer-tn-ref-p (tn-ref)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
-       (tn-ref-type tn-ref)
        (not (types-equal-or-intersect
              (tn-ref-type tn-ref)
              (specifier-type '(or fixnum
@@ -205,10 +259,19 @@
                                instance
                                character))))))
 
+(defun fixnum-or-other-pointer-tn-ref-p (tn-ref)
+  (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
+       (not (types-equal-or-intersect
+             (tn-ref-type tn-ref)
+             (specifier-type '(or #+64-bit single-float
+                               function
+                               list
+                               instance
+                               character))))))
+
 (defun not-nil-tn-ref-p (tn-ref)
-  (and (tn-ref-type tn-ref)
-       (not (types-equal-or-intersect (tn-ref-type tn-ref)
-                                      (specifier-type '(eql nil))))))
+  (not (types-equal-or-intersect (tn-ref-type tn-ref)
+                                 (specifier-type '(eql nil)))))
 
 (defun stack-consed-p (object)
   (let ((write (sb-c::tn-writes object))) ; list of write refs
@@ -304,10 +367,13 @@
   (declare (type (unsigned-byte 8) rank))
   (logand (1- rank) array-rank-mask))
 
-(defun compute-object-header (nwords widetag)
-  (let ((array-header-p
-         (or (= widetag simple-array-widetag)
-             (>= widetag complex-base-string-widetag))))
+(defun compute-object-header (nwords widetag-or-metadata)
+  (let* ((widetag (if (typep widetag-or-metadata '(or wrapper defstruct-description))
+                      instance-widetag
+                      widetag-or-metadata))
+         (array-header-p
+          (or (= widetag simple-array-widetag)
+              (>= widetag complex-base-string-widetag))))
     (logior (if array-header-p
                 (let ((rank (- nwords array-dimensions-offset)))
                   (ash (encode-array-rank rank) array-rank-position))
@@ -316,6 +382,10 @@
                   (t (ash (1- nwords) (length-field-shift widetag)))))
             widetag)))
 
+;;; Convert # of "big digits" (= words, sometimes called "limbs") to a header value.
+(defmacro bignum-header-for-length (n)
+  (logior (ash n n-widetag-bits) bignum-widetag))
+
 (defmacro id-bits-offset ()
   (let ((slot (get-dsd-index layout sb-kernel::id-word0)))
     (ash (+ sb-vm:instance-slots-offset slot) sb-vm:word-shift)))
