@@ -409,15 +409,19 @@
 ;;; the declared type for this variable
 (define-info-type (:variable :type)
   :type-spec ctype
-  :default #+sb-xc-host (lambda (x)
-                          (declare (special *universal-type*) (ignore x))
-                          *universal-type*)
-           #-sb-xc-host *universal-type*)
+  :default (lambda (name)
+             (declare (ignore name)
+                      #+sb-xc-host (special *universal-type*))
+             *universal-type*))
 
 ;;; where this type and kind information came from
 (define-info-type (:variable :where-from)
   :type-spec (member :declared :assumed :defined) :default :assumed)
 
+;;; a list of forward references to this constant.
+(define-info-type (:variable :forward-references)
+  :type-spec list)
+
 ;;; the macro-expansion for symbol-macros
 (define-info-type (:variable :macro-expansion) :type-spec t)
 
@@ -570,6 +574,13 @@
 (define-info-type (:source-location :declaration) :type-spec t)
 (define-info-type (:source-location :alien-type) :type-spec t)
 
+;;; If we used the maximum number of IDs available, a package gets no ID.
+;;; Any symbols in that package must use SYMBOL-DBINFO for their package.
+;;; Technically we can't store NIL, because that would be package ID 0,
+;;; i.e. directly represented in the symbol, but this type spec has to be
+;;; correct for what INFO can return, not what it may store.
+(define-info-type (:symbol :package) :type-spec (or package null))
+
 (!defun-from-collected-cold-init-forms !info-type-cold-init)
 
 #-sb-xc-host
