@@ -37,22 +37,6 @@
   (tagify alloc-tn rsp-tn lowtag)
   (values))
 
-#+nil ; was #+avx2
-(defun avx-registers-used-p ()
-  (or (when (and #+sb-xc-host (boundp '*component-being-compiled*))
-        (let ((comp (component-info *component-being-compiled*)))
-          (or (sb-c::ir2-component-avx2-used-p comp)
-              (flet ((used-p (tn)
-                       (do ((tn tn (sb-c::tn-next tn)))
-                           ((null tn))
-                         (when (sc-is tn ymm-reg
-                                      int-avx2-reg
-                                      double-avx2-reg single-avx2-reg)
-                           (return-from avx-registers-used-p
-                             (setf (sb-c::ir2-component-avx2-used-p comp) t))))))
-                (used-p (sb-c::ir2-component-normal-tns comp))
-                (used-p (sb-c::ir2-component-wired-tns comp))))))))
-
 (defun alloc-unboxed-p (type)
   (case type
     ((unboxed-array
@@ -181,6 +165,11 @@
              (inst .skip 8 :long-nop)))
       (emit-label skip-instrumentation))))
 
+;;; An arbitrary marker for the cons primitive-type, not to be confused
+;;; with the CONS-TYPE in our type-algebraic sense. Mostly just informs
+;;; the allocator to use cons_tlab.
+(defconstant +cons-primtype+ list-pointer-lowtag)
+
 ;;; Emit code to allocate an object with a size in bytes given by
 ;;; SIZE into ALLOC-TN. The size may be an integer of a TN.
 ;;; NODE may be used to make policy-based decisions.
@@ -193,7 +182,7 @@
 ;;; 1. what to allocate: type, size, lowtag describe the object
 ;;; 2. where to put the result
 ;;; 3. node (for determining immobile-space-p) and a scratch register or two
-(defun allocation (type size lowtag alloc-tn node temp thread-temp)
+(defun allocation (type size lowtag alloc-tn node temp thread-temp &key overflow)
   (declare (ignorable thread-temp))
   (flet ((fallback (size)
            ;; Call an allocator trampoline and get the result in the proper register.
@@ -205,19 +194,26 @@
                   (inst push alloc-tn))
                  (t
                   (inst push size)))
-           (invoke-asm-routine 'call (if (eq type 'list) 'list-alloc-tramp 'alloc-tramp) node t)
+           (invoke-asm-routine 'call
+                               (if (eql type +cons-primtype+) 'list-alloc-tramp 'alloc-tramp)
+                               node t)
            (inst pop alloc-tn)))
     (let* ((NOT-INLINE (gen-label))
            (DONE (gen-label))
-           (free-pointer #+sb-thread (thread-slot-ea thread-boxed-tlab-slot #+gs-seg thread-temp)
-                         #-sb-thread (ea boxed-region))
+           (free-pointer #+sb-thread (thread-slot-ea (if (eql type +cons-primtype+)
+                                                         thread-cons-tlab-slot
+                                                         thread-mixed-tlab-slot)
+                                                     #+gs-seg thread-temp)
+                         #-sb-thread (ea (if (eql type +cons-primtype+)
+                                             cons-region mixed-region)))
            (end-addr (ea (sb-x86-64-asm::ea-segment free-pointer)
                          (+ n-word-bytes (ea-disp free-pointer))
                          (ea-base free-pointer))))
       (cond ((typep size `(integer ,large-object-size))
              ;; large objects will never be made in a per-thread region
-             (fallback size)
-             (when (/= lowtag 0) (inst or :byte alloc-tn lowtag)))
+             (cond (overflow (funcall overflow))
+                   (t (fallback size)
+                      (when (/= lowtag 0) (inst or :byte alloc-tn lowtag)))))
             ((and (tn-p size) (location= size alloc-tn))
              (aver (and temp (not (location= temp size))))
              (inst mov temp free-pointer)
@@ -231,8 +227,9 @@
              (assemble (:elsewhere)
                (emit-label NOT-INLINE)
                (inst sub temp alloc-tn) ; new-free-ptr - old-free-ptr = size
-               (fallback temp)
-               (inst jmp DONE)))
+               (cond (overflow (funcall overflow))
+                     (t (fallback temp)
+                        (inst jmp DONE)))))
             (t
              ;; fixed-size allocation whose size fits in an imm32 can be done
              ;; with only one register, the ALLOC-TN. If it doesn't fit in imm32,
@@ -258,14 +255,17 @@
                           (t
                            ;; SUB can compute the result and tagify it.
                            ;; The fallback also has to tagify.
-                           (inst add alloc-tn (+ (- size) lowtag))
+                           (let ((bias (+ (- size) lowtag)))
+                             (if (= bias -1) (inst dec alloc-tn) (inst add alloc-tn bias)))
                            (emit-label DONE)))))
              (assemble (:elsewhere)
                (emit-label NOT-INLINE)
-               (fallback size)
-               (when (and (/= lowtag 0) (not temp) (not (tn-p size)))
-                 (inst or :byte alloc-tn lowtag))
-               (inst jmp DONE))))))
+               (cond (overflow (funcall overflow))
+                     (t
+                      (fallback size)
+                      (when (and (/= lowtag 0) (not temp) (not (tn-p size)))
+                        (inst or :byte alloc-tn lowtag))
+                      (inst jmp DONE))))))))
   t)
 
 ;;; Allocate an other-pointer object of fixed NWORDS with a single-word
@@ -312,7 +312,7 @@
                        temp)
                       (t
                        (encode-value-if-immediate ,tn)))))
-                (storew* reg ,list ,slot ,lowtag (not stack-allocate-p) temp))))
+                (storew reg ,list ,slot ,lowtag temp))))
 
 (define-vop (cons)
   (:args (car :scs (any-reg descriptor-reg constant immediate))
@@ -326,11 +326,11 @@
     (let ((stack-allocate-p (node-stack-allocate-p node))
           (nbytes (* cons-size n-word-bytes)))
       (unless stack-allocate-p
-        (instrument-alloc 'list nbytes node (list temp alloc) thread-tn))
+        (instrument-alloc +cons-primtype+ nbytes node (list temp alloc) thread-tn))
       (pseudo-atomic (:elide-if stack-allocate-p :thread-tn thread-tn)
         (if stack-allocate-p
             (stack-allocation nbytes 0 alloc)
-            (allocation 'list nbytes 0 alloc node temp thread-tn))
+            (allocation +cons-primtype+ nbytes 0 alloc node temp thread-tn))
         (store-slot car alloc cons-car-slot 0)
         (store-slot cdr alloc cons-cdr-slot 0)
         (if (location= alloc result)
@@ -349,11 +349,10 @@
   (:translate acons)
   (:policy :fast-safe)
   (:generator 10
-    (let ((stack-allocate-p nil)
-          (nbytes (* cons-size 2 n-word-bytes)))
-      (instrument-alloc 'list nbytes node (list temp alloc) thread-tn)
+    (let ((nbytes (* cons-size 2 n-word-bytes)))
+      (instrument-alloc +cons-primtype+ nbytes node (list temp alloc) thread-tn)
       (pseudo-atomic (:thread-tn thread-tn)
-        (allocation 'list nbytes 0 alloc node temp thread-tn)
+        (allocation +cons-primtype+ nbytes 0 alloc node temp thread-tn)
         (store-slot tail alloc cons-cdr-slot 0)
         (inst lea temp (ea (+ 16 list-pointer-lowtag) alloc))
         (store-slot temp alloc cons-car-slot 0)
@@ -381,11 +380,11 @@
     (let ((stack-allocate-p (node-stack-allocate-p node))
           (nbytes (* cons-size 2 n-word-bytes)))
       (unless stack-allocate-p
-        (instrument-alloc 'list nbytes node (list temp alloc) thread-tn))
+        (instrument-alloc +cons-primtype+ nbytes node (list temp alloc) thread-tn))
       (pseudo-atomic (:elide-if stack-allocate-p :thread-tn thread-tn)
         (if stack-allocate-p
             (stack-allocation nbytes 0 alloc)
-            (allocation 'list nbytes 0 alloc node temp thread-tn))
+            (allocation +cons-primtype+ nbytes 0 alloc node temp thread-tn))
         (store-slot car alloc cons-car-slot 0)
         (store-slot cadr alloc (+ 2 cons-car-slot) 0)
         (store-slot cddr alloc (+ 2 cons-cdr-slot) 0)
@@ -396,7 +395,7 @@
             (inst lea result (ea list-pointer-lowtag alloc)))))))
 
 (define-vop (list)
-  (:args (things :more t :scs (descriptor-reg constant immediate)))
+  (:args (things :more t :scs (descriptor-reg any-reg constant immediate)))
   (:temporary (:sc unsigned-reg) ptr temp)
   (:temporary (:sc unsigned-reg :to (:result 0) :target result) res)
   #+gs-seg (:temporary (:sc unsigned-reg :offset 15) thread-tn)
@@ -408,11 +407,11 @@
     (let ((stack-allocate-p (node-stack-allocate-p node))
           (size (* (pad-data-block cons-size) cons-cells)))
       (unless stack-allocate-p
-        (instrument-alloc 'list size node (list ptr temp) thread-tn))
+        (instrument-alloc +cons-primtype+ size node (list ptr temp) thread-tn))
       (pseudo-atomic (:elide-if stack-allocate-p :thread-tn thread-tn)
         (if stack-allocate-p
             (stack-allocation size list-pointer-lowtag res)
-            (allocation 'list size list-pointer-lowtag res node temp thread-tn))
+            (allocation +cons-primtype+ size list-pointer-lowtag res node temp thread-tn))
         (move ptr res)
         (dotimes (i (1- cons-cells))
           (store-slot (tn-ref-tn things) ptr)
@@ -424,10 +423,9 @@
                (setf things (tn-ref-across things))
                (store-slot (tn-ref-tn things) ptr cons-cdr-slot list-pointer-lowtag))
               (t
-               (storew* nil-value ptr cons-cdr-slot list-pointer-lowtag
-                        (not stack-allocate-p))))))
-    (aver (null (tn-ref-across things)))
-    (move result res)))
+               (storew nil-value ptr cons-cdr-slot list-pointer-lowtag)))
+        (aver (null (tn-ref-across things)))
+        (move result res)))))
 )
 
 ;;;; special-purpose inline allocators
@@ -751,9 +749,9 @@
 
   (define-vop (allocate-list-on-heap)
     (:args (length :scs (any-reg immediate))
-           (element :scs (any-reg descriptor-reg)
-                    :load-if (not (and (sc-is element immediate)
-                                       (eql (tn-value element) 0)))))
+           ;; Too bad we don't have an SC that implies actually a CPU immediate
+           ;; i.e. fits in an imm32 operand
+           (element :scs (any-reg descriptor-reg)))
     (:results (result :scs (descriptor-reg) :from :load))
     (:arg-types positive-fixnum *)
     (:policy :fast-safe)
@@ -764,11 +762,17 @@
       (let ((size (calc-size-in-bytes length tail))
             (entry (gen-label))
             (loop (gen-label))
-            (no-init
-             (and (sc-is element immediate) (eql (tn-value element) 0))))
-        (instrument-alloc 'list size node (list next limit) thread-tn)
+            (leave-pa (gen-label)))
+        (instrument-alloc +cons-primtype+ size node (list next limit) thread-tn)
         (pseudo-atomic (:thread-tn thread-tn)
-         (allocation 'list size list-pointer-lowtag result node limit thread-tn)
+         (allocation +cons-primtype+ size list-pointer-lowtag result node limit thread-tn
+                     :overflow (lambda ()
+                                 ;; Push C call args right-to-left
+                                 (inst push (if (integerp size) (constantize size) size))
+                                 (inst push (if (sc-is element immediate) (tn-value element) element))
+                                 (invoke-asm-routine 'call 'make-list node)
+                                 (inst pop result)
+                                 (inst jmp leave-pa)))
          (compute-end)
          (inst mov next result)
          (inst jmp entry)
@@ -777,12 +781,13 @@
          (emit-label ENTRY)
          (inst mov tail next)
          (inst add next (* 2 n-word-bytes))
-         (unless no-init ; don't bother writing zeros in the CARs
-           (storew element tail cons-car-slot list-pointer-lowtag))
+         (storew element tail cons-car-slot list-pointer-lowtag)
          (inst cmp next limit)
-         (inst jmp :ne loop))
-        (storew nil-value tail cons-cdr-slot list-pointer-lowtag))
-      done)))
+         (inst jmp :ne loop)
+         ;; still pseudo-atomic
+         (storew nil-value tail cons-cdr-slot list-pointer-lowtag)
+         (emit-label leave-pa)))
+      done))) ; label needed by calc-size-in-bytes
 
 #-immobile-space
 (define-vop (make-fdefn)
@@ -800,39 +805,42 @@
             result fdefn-raw-addr-slot other-pointer-lowtag)))
 
 (define-vop (make-closure)
-  ; (:args (function :to :save :scs (descriptor-reg)))
   (:info label length stack-allocate-p)
   (:temporary (:sc any-reg) temp)
   #+gs-seg (:temporary (:sc unsigned-reg :offset 15) thread-tn)
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 10
-   (let* ((words (+ length closure-info-offset)) ; including header
-          (bytes (pad-data-block words))
-          (header (logior (ash (1- words) n-widetag-bits) closure-widetag)))
-     (unless stack-allocate-p
-       (instrument-alloc closure-widetag bytes node (list result temp) thread-tn))
-     (pseudo-atomic (:elide-if stack-allocate-p :thread-tn thread-tn)
-       (if stack-allocate-p
-           (stack-allocation bytes fun-pointer-lowtag result)
-           (allocation nil bytes fun-pointer-lowtag result node temp thread-tn))
-       (storew* #-immobile-space header ; write the widetag and size
-                #+immobile-space        ; ... plus the layout pointer
-                (progn (inst mov temp header)
-                       (inst or temp #-sb-thread (static-symbol-value-ea 'function-layout)
-                                     #+sb-thread
-                                     (thread-slot-ea thread-function-layout-slot))
-                       temp)
-                result 0 fun-pointer-lowtag (not stack-allocate-p)))
-     ;; Finished with the pseudo-atomic instructions
-     (when label
-       ;; TODO: gencgc does not need EMIT-GC-STORE-BARRIER here, but other other GC strategies might.
-       (inst lea temp (rip-relative-ea label (ash simple-fun-insts-offset word-shift)))
-       (storew temp result closure-fun-slot fun-pointer-lowtag)
-       #+metaspace
-       (let ((origin (sb-assem::asmstream-data-origin-label sb-assem:*asmstream*)))
-         (inst lea temp (rip-relative-ea origin :code))
-         (storew temp result closure-code-slot fun-pointer-lowtag))))))
+    (let* ((words (+ length closure-info-offset)) ; including header
+           (bytes (pad-data-block words))
+           (header (logior (ash (1- words) n-widetag-bits) closure-widetag)))
+      (unless stack-allocate-p
+        (instrument-alloc closure-widetag bytes node (list result temp) thread-tn))
+      (pseudo-atomic (:elide-if stack-allocate-p :thread-tn thread-tn)
+        (if stack-allocate-p
+            (stack-allocation bytes fun-pointer-lowtag result)
+            (allocation nil bytes fun-pointer-lowtag result node temp thread-tn))
+        (storew* #-immobile-space header ; write the widetag and size
+                 #+immobile-space        ; ... plus the layout pointer
+                 (let ((layout #-sb-thread (static-symbol-value-ea 'function-layout)
+                               #+sb-thread (thread-slot-ea thread-function-layout-slot)))
+                   (cond ((typep header '(unsigned-byte 16))
+                          (inst mov temp layout)
+                          ;; emit a 2-byte constant, the low 4 of TEMP were zeroed
+                          (inst mov :word temp header))
+                         (t
+                          (inst mov temp header)
+                          (inst or temp layout)))
+                   temp)
+                 result 0 fun-pointer-lowtag (not stack-allocate-p)))
+      ;; Finished with the pseudo-atomic instructions
+      ;; TODO: gencgc does not need EMIT-GC-STORE-BARRIER here, but other other GC strategies might.
+      (inst lea temp (rip-relative-ea label (ash simple-fun-insts-offset word-shift)))
+      (storew temp result closure-fun-slot fun-pointer-lowtag)
+      #+metaspace
+      (let ((origin (sb-assem::asmstream-data-origin-label sb-assem:*asmstream*)))
+        (inst lea temp (rip-relative-ea origin :code))
+        (storew temp result closure-code-slot fun-pointer-lowtag)))))
 
 ;;; The compiler likes to be able to directly make value cells.
 (define-vop (make-value-cell)
@@ -863,16 +871,10 @@
           (inst lea result (ea tramp rip-tn))
           (inst mov result tramp)))))
 
-(define-vop (fixed-alloc)
-  (:args)
-  (:info name words type lowtag stack-allocate-p)
-  (:results (result :scs (descriptor-reg)))
-  (:temporary (:sc unsigned-reg) alloc-temp)
-  #+gs-seg (:temporary (:sc unsigned-reg :offset 15) thread-tn)
-  (:node-var node)
-  (:generator 50
-   (let* ((instancep (typep type 'wrapper)) ; is this an instance type?
-          (bytes (pad-data-block words)))
+(flet
+  ((alloc (name words type lowtag stack-allocate-p result
+                    &optional alloc-temp node
+                    &aux (bytes (pad-data-block words)))
     #+bignum-assertions
     (when (eq type bignum-widetag) (setq bytes (* bytes 2))) ; use 2x the space
     (progn name) ; possibly not used
@@ -884,28 +886,42 @@
       (if stack-allocate-p
           (stack-allocation bytes (if type 0 lowtag) result)
           (allocation nil bytes (if type 0 lowtag) result node alloc-temp thread-tn))
-      (when type
-        (let* ((widetag (if instancep instance-widetag type))
-               (header (compute-object-header words widetag)))
-          (if (or #+compact-instance-header
-                  (and (eq name '%make-structure-instance) stack-allocate-p))
+      (let ((header (compute-object-header words type)))
+        (cond #+compact-instance-header
+              ((and (eq name '%make-structure-instance) stack-allocate-p)
               ;; Write a :DWORD, not a :QWORD, because the high half will be
               ;; filled in when the layout is stored. Can't use STOREW* though,
               ;; because it tries to store as few bytes as possible,
               ;; where this instruction must write exactly 4 bytes.
-              (inst mov :dword (ea 0 result) header)
-              (storew* header result 0 0 (not stack-allocate-p)))
-          (inst or :byte result lowtag))))
-    (when instancep ; store its layout
-      (inst mov :dword (ea (+ 4 (- lowtag)) result)
-            (make-fixup type :layout))))))
+               (inst mov :dword (ea 0 result) header))
+              (t
+               (storew* header result 0 0 (not stack-allocate-p)))))
+      ;; GC can make the best choice about placement if it has a layout.
+      ;; Of course with conservative GC the object will be pinned anyway,
+      ;; but still, always having a layout is a good thing.
+      (when (typep type 'wrapper) ; store its layout, while still in pseudo-atomic
+        (inst mov :dword (ea 4 result) (make-fixup type :layout)))
+      (inst or :byte result lowtag))))
+  ;; DX is strictly redundant in these 2 vops, but they're written this way
+  ;; so that backends can choose to use a single vop for both.
+  (define-vop (fixed-alloc)
+    (:info name words type lowtag dx)
+    (:results (result :scs (descriptor-reg)))
+    (:temporary (:sc unsigned-reg) alloc-temp)
+    #+gs-seg (:temporary (:sc unsigned-reg :offset 15) thread-tn)
+    (:node-var node)
+    (:generator 50 (alloc name words type lowtag dx result alloc-temp node)))
+  (define-vop (sb-c::fixed-alloc-to-stack)
+    (:info name words type lowtag dx)
+    (:results (result :scs (descriptor-reg)))
+    (:generator 50 (alloc name words type lowtag dx result))))
 
 ;;; Allocate a non-vector variable-length object.
 ;;; Exactly 4 allocators are rendered via this vop:
 ;;;  BIGNUM               (%ALLOCATE-BIGNUM)
 ;;;  FUNCALLABLE-INSTANCE (%MAKE-FUNCALLABLE-INSTANCE)
 ;;;  CLOSURE              (%ALLOC-CLOSURE)
-;;;  INSTANCE             (%MAKE-INSTANCE)
+;;;  INSTANCE             (%MAKE-INSTANCE,%MAKE-INSTANCE/MIXED)
 ;;; WORDS accounts for the mandatory slots *including* the header.
 ;;; EXTRA is the variable payload, also measured in words.
 (define-vop (var-alloc)
@@ -946,11 +962,11 @@
               (allocation nil bytes lowtag result node alloc-temp thread-tn)
               (storew header result 0 lowtag))))))
 
+#+immobile-space
 (macrolet ((c-call (name)
              `(let ((c-fun (make-fixup ,name :foreign)))
                 (inst call (cond ((sb-c::code-immobile-p node) c-fun)
                                  (t (progn (inst mov rax c-fun) rax)))))))
-#+immobile-space
 (define-vop (alloc-immobile-fixedobj)
   (:args (size-class :scs (any-reg) :target c-arg1)
          (nwords :scs (any-reg) :target c-arg2)
@@ -972,19 +988,73 @@
      (c-call "alloc_immobile_fixedobj")
      (move result rax))))
 
-(define-vop (alloc-dynamic-space-code)
-  (:args (total-words :scs (signed-reg) :target c-arg1))
-  (:temporary (:sc unsigned-reg :from (:argument 0) :to :eval :offset rdi-offset) c-arg1)
-  (:temporary (:sc unsigned-reg :from :eval :to (:result 0) :offset rax-offset) rax)
+;;; Timing test:
+;;; Dynamic-space allocation:
+;;; * (defun f (n) (dotimes (i n) (make-symbol "b")))
+;;; * (time (f 500000))
+;;; Evaluation took: 0.004 seconds of real time
+;;; Immobile-space:
+;;; * (defun f (n) (dotimes (i n) (sb-vm::make-immobile-symbol "b")))
+;;; Evaluation took: 0.043 seconds of real time
+;;; With vop:  0.028 seconds of real time
+
+(eval-when (:compile-toplevel)
+  (aver (evenp symbol-size))) ; assumptions in the code below
+
+(define-vop (fast-alloc-immobile-symbol)
   (:results (result :scs (descriptor-reg)))
-  (:node-var node)
-  (:generator 50
-   (inst mov c-arg1 total-words)
-   ;; RSP needn't be restored because the allocators all return immediately
-   ;; which has that effect
-   (inst and rsp-tn -16)
-   (pseudo-atomic () (c-call "alloc_code_object"))
-   ;; RESULT is a tagged ptr. MOV doesn't need to be inside the PSEUDO-ATOMIC.
-   (inst mov result rax)))
+  (:temporary (:sc unsigned-reg :offset rax-offset) rax)
+  (:temporary (:sc unsigned-reg :offset rbx-offset) rbx)
+  (:temporary (:sc unsigned-reg :offset rcx-offset) rcx)
+  (:temporary (:sc unsigned-reg) header)
+  (:generator 1
+    ;; fixedobj_pages linkage entry: 1 PTE per page, 12-byte struct
+    (inst mov rbx (ea (make-fixup "fixedobj_pages" :foreign-dataref)))
+    ;; fixedobj_page_hint: 1 hint per sizeclass. C type = uint32_t
+    (inst mov rax (ea (make-fixup "fixedobj_page_hint" :foreign-dataref)))
+    (inst mov rbx (ea rbx)) ; get the base of the fixedobj_pages array
+    ;; This has to be pseudoatomic as soon as the page hint is loaded.
+    ;; Consider the situation where there is exactly one symbol on a page that
+    ;; is almost all free, and the symbol page hint points to that page.
+    ;; If GC occurs, it might dispose that symbol, resetting the page attributes
+    ;; and page hint. It would be an error to allocate to that page
+    ;; because it is no longer a page of symbols but rather a free page.
+    ;; There is no way to inform GC that we are currently looking at a page
+    ;; in anticipation of allocating to it.
+    (pseudo-atomic ()
+     (assemble ()
+       (inst mov :dword rax (ea 4 rax)) ; rax := fixedobj_page_hint[1] (sizeclass=SYMBOL)
+       (inst test :dword rax rax)
+       (inst jmp :z FAIL) ; fail if hint page is 0
+       (inst lea rbx (ea rbx rax 8))  ; rbx := &fixedobj_pages[hint].free_index
+       ;; compute fixedobj_page_address(hint) into RAX
+       (inst mov rcx (ea (make-fixup "FIXEDOBJ_SPACE_START" :foreign-dataref)))
+       (inst shl rax (integer-length (1- immobile-card-bytes)))
+       (inst add rax (ea rcx))
+       ;; load the page's free pointer
+       (inst mov :dword rcx (ea rbx)) ; rcx := fixedobj_pages[hint].free_index
+       ;; fail if allocation would overrun the page
+       (inst cmp :dword rcx (- immobile-card-bytes (* symbol-size n-word-bytes)))
+       (inst jmp :a FAIL)
+       ;; compute address of the allegedly free memory block into RESULT
+       (inst lea result (ea rcx rax)) ; free_index + page_base
+       ;; read the potential symbol header
+       (inst mov rax (ea result))
+       (inst test :dword rax 1)
+       (inst jmp :nz FAIL) ; not a fixnum implies already taken
+       ;; try to claim this word of memory
+       (inst mov header (logior (ash (1- symbol-size) n-widetag-bits) symbol-widetag))
+       (inst cmpxchg :lock (ea result) header)
+       (inst jmp :ne FAIL) ; already taken
+       ;; compute new free_index = spacing + old header + free_index
+       (inst lea :dword rax (ea (* symbol-size n-word-bytes) rax rcx))
+       (inst mov :dword (ea rbx) rax) ; store new free_index
+       ;; set the low bit of the 'gens' field
+       (inst or :lock :byte (ea 7 rbx) 1) ; 7+rbx = &fixedobj_pages[i].attr.parts.gens_
+       (inst or :byte result other-pointer-lowtag) ; make_lispobj()
+       (inst jmp OUT)
+       FAIL
+       (inst mov result nil-value)
+       OUT))))
 
 ) ; end MACROLET
