@@ -38,6 +38,8 @@
   (:info name offset lowtag)
   (:results)
   (:vop-var vop)
+  (:node-var node)
+  (:args-var args)
   (:temporary (:sc unsigned-reg) val-temp)
   (:generator 1
     (cond #+ubsan
@@ -49,10 +51,19 @@
            (inst mov :dword (vector-len-ea object)
                  (or (encode-value-if-immediate value) value)))
           (t
-           ;; gencgc does not need to emit the barrier for constructors
-           (unless (member name '(%make-structure-instance make-weak-pointer
-                                  %make-ratio %make-complex))
-             (emit-gc-store-barrier object nil val-temp (vop-nth-arg 1 vop) value))
+           (let* ((value-tn (tn-ref-tn (tn-ref-across args)))
+                  (prim-type (sb-c::tn-primitive-type value-tn))
+                  (scs (and prim-type
+                            (sb-c::primitive-type-scs prim-type))))
+             (unless (and
+                      ;; Can this TN be boxed after the allocator?
+                      (or (singleton-p scs)
+                          (not (member descriptor-reg-sc-number scs)))
+                      (or
+                       ;; gencgc does not need to emit the barrier for constructors
+                       (eq name :allocator)
+                       (sb-c::set-slot-old-p node)))
+               (emit-gc-store-barrier object nil val-temp (vop-nth-arg 1 vop) value)))
            (gen-cell-set (object-slot-ea object offset lowtag) value val-temp)))))
 
 (define-vop (compare-and-swap-slot)
@@ -82,24 +93,99 @@
   (:generator 1
     (inst mov result (unbound-marker-bits))))
 
+(defmacro emit-symbol-write-barrier (sym &rest rest)
+  ;; IMMEDIATE sc means that the symbol is static or immobile.
+  ;; Static symbols are roots, and immobile symbols use page fault handling.
+  `(unless (sc-is ,sym immediate) (emit-gc-store-barrier ,sym ,@rest)))
+
 (define-vop (%set-symbol-global-value)
-  (:args (object :scs (descriptor-reg immediate))
+  (:args (symbol :scs (descriptor-reg immediate))
          (value :scs (descriptor-reg any-reg immediate)))
   (:policy :fast-safe)
   (:temporary (:sc unsigned-reg) val-temp)
   (:vop-var vop)
   (:generator 4
-    (emit-gc-store-barrier object nil val-temp (vop-nth-arg 1 vop) value)
-    (gen-cell-set (if (sc-is object immediate)
-                      (symbol-slot-ea (tn-value object) symbol-value-slot)
-                      (object-slot-ea object symbol-value-slot other-pointer-lowtag))
+    (emit-symbol-write-barrier symbol nil val-temp (vop-nth-arg 1 vop) value)
+    (gen-cell-set (if (sc-is symbol immediate)
+                      (symbol-slot-ea (tn-value symbol) symbol-value-slot)
+                      (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))
                   value val-temp)))
 
+;;; This does not resolve the TLS-INDEX at load-time, because we don't want to
+;;; waste TLS indices for symbols that may never get thread-locally bound.
+;;; So instead we make a fixup to the address of the 4-byte TLS field in the symbol.
+(defun symbol-tls-index-ea (symbol)
+  (ea (make-fixup (tn-value symbol) :immobile-symbol (- 4 other-pointer-lowtag))))
+
+(defun get-symbol-value-slot-ea (ea-tn symbol)
+  (if (sc-is symbol immediate)
+      (inst mov ea-tn (make-fixup (tn-value symbol) :immobile-symbol
+                                  (- (ash symbol-value-slot word-shift) other-pointer-lowtag)))
+      (inst lea ea-tn (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))))
+
+;;; As implemented for x86-64, SET performs at most one conditional branch and no
+;;; unconditional jump, contrasting with the other common approach of branching around
+;;; a store to either the TLS or the global value, followed by an unconditional jump
+;;; out, and in the middle doing the opposite of whichever store was jumped over.
+;;; (see the arm64, ppc64, and riscv implementations)
+;;; I would surmise that one unconditional branch is preferable.
+;;; Slightly in favor of that claim is that if you run this through a C compiler:
+;;;   void f1(long val, int test, long *a, long *b) {
+;;;       if (test) *a = val; else *b = val;
+;;;   }
+;;; it generates the same code as if you had written "*(test?a:b) = val;"
+;;; So while that's not 100% analagous to this situation here, it does imply
+;;; slight favoritism for branch-free code. It might be possible to use CMOV
+;;; if we don't need to emit the GC barrier such as when storing a fixnum.
+;;; On the other hand, Torvalds has ranted against writing branchless code
+;;; for its own sake (https://yarchive.net/comp/linux/cmov.html)
+;;;
+;;; FIXME: quite likely this vop should exist in several variants, for a known
+;;; symbol, variable symbol, etc. As it stands, we can see that a symbol is
+;;; always-thread-local only if immobile-space is in use, wherein the symbol's SC
+;;; is immediate.
+#+sb-thread
+(define-vop (set)
+  (:args (symbol :scs (descriptor-reg immediate))
+         (value :scs (descriptor-reg any-reg immediate)))
+  (:temporary (:sc unsigned-reg) val-temp cell)
+  #+gs-seg (:temporary (:sc unsigned-reg) thread-temp)
+  (:vop-var vop)
+  (:generator 4
+    (if (and (sc-is symbol immediate)
+             (eq (info :variable :wired-tls (tn-value symbol)) :always-thread-local))
+        ;; We never need the GC barrier for TLS.  I think it would be preferable
+        ;; to resolve this in IR1, maybe turning it into SET-TLS-VALUE.
+        (gen-cell-set (ea (make-fixup (tn-value symbol) :symbol-tls-index) thread-tn)
+                      value val-temp)
+        (let ((store (gen-label)))
+          (cond ((and (sc-is symbol immediate)
+                      (info :variable :wired-tls (tn-value symbol)))
+                 ;; The TLS index is arbitrary but known to be nonzero,
+                 ;; so we can resolve the displacement of the thread-local value
+                 ;; at load-time, saving one instruction over the general case.
+                 (inst lea cell (ea (make-fixup (tn-value symbol) :symbol-tls-index)
+                                    thread-tn)))
+                (t
+                 ;; These MOVs look the same, but when the symbol is immediate, this is
+                 ;; a load from an absolute address. Needless to say, the names of these
+                 ;; accessor macros are arbitrary - the difference is not very apparent.
+                 (inst mov :dword cell (if (sc-is symbol immediate)
+                                           (symbol-tls-index-ea symbol)
+                                           (tls-index-of symbol)))
+                 (inst add cell thread-tn)))
+          (inst cmp :qword (ea cell) no-tls-value-marker)
+          (inst jmp :ne STORE)
+          (emit-symbol-write-barrier symbol nil val-temp (vop-nth-arg 1 vop) value)
+          (get-symbol-value-slot-ea cell symbol)
+          (emit-label STORE)
+          (gen-cell-set (ea cell) value val-temp)))))
+
 (define-vop (fast-symbol-global-value)
   (:args (object :scs (descriptor-reg immediate)))
   (:results (value :scs (descriptor-reg any-reg)))
   (:policy :fast)
-  (:translate sym-global-val)
+  (:translate symbol-global-value)
   (:generator 4
     (cond ((sc-is object immediate)
            (inst mov value (symbol-slot-ea (tn-value object) symbol-value-slot)))
@@ -108,7 +194,7 @@
 
 (define-vop (symbol-global-value)
   (:policy :fast-safe)
-  (:translate sym-global-val)
+  (:translate symbol-global-value)
   (:args (object :scs (descriptor-reg) :to (:result 1)))
   (:results (value :scs (descriptor-reg any-reg)))
   (:vop-var vop)
@@ -131,34 +217,40 @@
       `(%cas-symbol-global-value symbol old new)
       (sb-c::give-up-ir1-transform)))
 
-(macrolet (;; Logic common to thread-aware SET and CAS. CELL is assigned
-           ;; to the location that should be accessed to modify SYMBOL's
-           ;; value either in the TLS or the symbol's value slot as follows:
-           ;; (1) make it look as if the TLS cell were a symbol by biasing
-           ;;     upward by other-pointer-lowtag less 1 word.
-           ;; (2) conditionally make CELL point to the symbol itself
-           (compute-virtual-symbol ()
-             `(progn
-                (inst mov :dword cell (tls-index-of symbol))
-                #+gs-seg (inst rdgsbase thread-temp)
-                (inst lea cell
-                      (ea (- other-pointer-lowtag (ash symbol-value-slot word-shift))
-                          #+gs-seg thread-temp
-                          #-gs-seg thread-tn
-                          cell))
-                (inst cmp :dword (symbol-value-slot-ea cell) ; TLS reference
-                      no-tls-value-marker-widetag)
-                (inst cmov :e cell symbol))) ; now possibly get the symbol
-           (access-wired-tls-val (sym) ; SYM is a symbol
+(macrolet ((access-wired-tls-val (sym) ; SYM is a symbol
              `(thread-tls-ea (load-time-tls-offset ,sym)))
            (symbol-value-slot-ea (sym) ; SYM is a TN
              `(ea (- (* symbol-value-slot n-word-bytes) other-pointer-lowtag)
-                  ,sym)))
+                  ,sym))
+           (load-oldval ()
+             `(if (sc-is old immediate)
+                  (inst mov rax (encode-value-if-immediate old))
+                  (move rax old))))
 
-  (define-vop (%compare-and-swap-symbol-value)
+(define-vop (%cas-symbol-global-value)
+    (:translate %cas-symbol-global-value)
+    (:args (symbol :scs (descriptor-reg immediate) :to (:result 0))
+           (old :scs (descriptor-reg any-reg constant immediate))
+           (new :scs (descriptor-reg any-reg)))
+    ;; RAX is the temp for computing the card mark, so it has to conflict
+    ;; with OLD and therefore the default lifetime spec is fine
+    (:temporary (:sc descriptor-reg :offset rax-offset :to (:result 0)) rax)
+    (:results (result :scs (descriptor-reg any-reg)))
+    (:policy :fast-safe)
+    (:vop-var vop)
+    (:generator 10
+      (emit-symbol-write-barrier symbol nil rax (vop-nth-arg 2 vop) new)
+      (load-oldval)
+      (inst cmpxchg :lock (if (sc-is symbol immediate)
+                              (symbol-slot-ea (tn-value symbol) symbol-value-slot)
+                              (symbol-value-slot-ea symbol))
+            new)
+      (move result rax)))
+
+(define-vop (%compare-and-swap-symbol-value)
     (:translate %compare-and-swap-symbol-value)
-    (:args (symbol :scs (descriptor-reg) :to (:result 0))
-           (old :scs (descriptor-reg any-reg) :target rax)
+    (:args (symbol :scs (descriptor-reg immediate) :to (:result 0))
+           (old :scs (descriptor-reg any-reg constant immediate) :target rax)
            (new :scs (descriptor-reg any-reg)))
     (:temporary (:sc descriptor-reg :offset rax-offset
                  :from (:argument 1) :to (:result 0)) rax)
@@ -167,6 +259,7 @@
     (:results (result :scs (descriptor-reg any-reg)))
     (:policy :fast-safe)
     (:vop-var vop)
+    (:node-var node)
     (:generator 15
     ;; This code has two pathological cases: NO-TLS-VALUE-MARKER
     ;; or UNBOUND-MARKER as NEW: in either case we would end up
@@ -174,63 +267,31 @@
     ;; Even worse: don't supply old=NO-TLS-VALUE with a symbol whose
     ;; tls-index=0, because that would succeed, assigning NEW to each
     ;; symbol in existence having otherwise no thread-local value.
-      ;; Possible optimization: don't frob the card mark when storing into TLS
-      (emit-gc-store-barrier symbol nil cell (vop-nth-arg 2 vop) new)
-      (let ((unbound (generate-error-code vop 'unbound-symbol-error symbol)))
-        #+sb-thread (progn (compute-virtual-symbol)
-                            (move rax old)
-                            (inst cmpxchg :lock (symbol-value-slot-ea cell) new))
-        #-sb-thread (progn (move rax old)
-                            ;; is the :LOCK is necessary?
-                            (inst cmpxchg :lock (symbol-value-slot-ea symbol) new))
+      #+sb-thread (progn
+      (inst mov :dword cell (if (sc-is symbol immediate)
+                                (symbol-tls-index-ea symbol)
+                                (tls-index-of symbol)))
+      (inst add cell thread-tn)
+      (inst cmp :qword (ea cell) no-tls-value-marker)
+      (inst jmp :ne CAS))
+      ;; GLOBAL. All logic that follows is for both + and - sb-thread
+      (emit-symbol-write-barrier symbol nil cell (vop-nth-arg 2 vop) new)
+      (get-symbol-value-slot-ea cell symbol)
+      CAS
+      (load-oldval)
+      (inst cmpxchg :lock (ea cell) new)
+      ;; FIXME: if :ALWAYS-BOUND then elide the BOUNDP check.
+      ;; But we don't accept a constant or immediate, so how to know
+      ;; what symbol is being CASed? I kind of feel like whether to perform
+      ;; the check ought to be supplied as codegen info so that we don't
+      ;; conflate storage-class of an arg with whether we could elide the check.
+      (unless (policy node (= safety 0))
         (inst cmp :byte rax unbound-marker-widetag)
-        (inst jmp :e unbound)
-        (move result rax))))
-
-  (define-vop (%cas-symbol-global-value)
-    (:translate %cas-symbol-global-value)
-    (:args (symbol :scs (descriptor-reg immediate) :to (:result 0))
-           (old :scs (descriptor-reg any-reg) #|:target rax|#)
-           (new :scs (descriptor-reg any-reg)))
-    ;; if OLD were LOCATION= to RAX then we'd clobber OLD
-    ;; while computing the EA for the barrier.
-    (:temporary (:sc descriptor-reg :offset rax-offset
-                 #|:from (:argument 1)|# :to (:result 0)) rax)
-    (:results (result :scs (descriptor-reg any-reg)))
-    (:policy :fast-safe)
-    (:vop-var vop)
-    (:generator 10
-      (emit-gc-store-barrier symbol nil rax (vop-nth-arg 2 vop) new)
-      (move rax old)
-      (inst cmpxchg :lock
-            (if (sc-is symbol immediate)
-                (symbol-slot-ea (tn-value symbol) symbol-value-slot)
-                (symbol-value-slot-ea symbol))
-            new)
+        (inst jmp :e (generate-error-code vop 'unbound-symbol-error symbol)))
       (move result rax)))
 
   #+sb-thread
   (progn
-    ;; TODO: SET could be shorter for any known wired-tls symbol.
-    ;; Note that the 32-bit x86 code prefers to use branching code here, where it accesses
-    ;; either the symbol's slot or the segment-relative absolute displacement to the TLS.
-    ;; This prefers CMOV, which means we always need the thread's address in a GPR.
-    (define-vop (set)
-      (:args (symbol :scs (descriptor-reg))
-             (value :scs (descriptor-reg any-reg immediate)))
-      (:temporary (:sc descriptor-reg) cell)
-      (:temporary (:sc unsigned-reg) val-temp)
-      #+gs-seg (:temporary (:sc unsigned-reg) thread-temp)
-      (:vop-var vop)
-      (:generator 4
-        ;; Possible optimization: don't frob the card mark when storing into TLS
-        (emit-gc-store-barrier symbol nil val-temp (vop-nth-arg 1 vop) value)
-        ;; Compute the address into which to store. CMOV can only move into
-        ;; a register, so we can't conditionally move into the TLS and
-        ;; conditionally move in the opposite flag sense to the symbol.
-        (compute-virtual-symbol)
-        (gen-cell-set (symbol-value-slot-ea cell) value val-temp)))
-
     ;; This code is tested by 'codegen.impure.lisp'
     (defun emit-symeval (value symbol symbol-reg check-boundp vop)
       (let* ((known-symbol-p (sc-is symbol constant immediate))
@@ -272,7 +333,7 @@
               (setq symbol-reg symbol)))
 
            ;; Load the global value if the TLS value didn't exist
-           (inst cmp :dword value no-tls-value-marker-widetag)
+           (inst cmp :qword value no-tls-value-marker)
            (inst cmov :e value
                  (if (and known-symbol-p (sc-is symbol immediate))
                      (symbol-slot-ea known-symbol symbol-value-slot) ; MOV Rxx, imm32
@@ -299,10 +360,9 @@
                                 symbol-reg))))
             RETRY))))
 
-  ;; With Symbol-Value, we check that the value isn't the trap object. So
-  ;; Symbol-Value of NIL is NIL.
+  ;; With Symbol-Value, we check that the value isn't the trap object.
     (define-vop (symbol-value)
-      (:translate symeval)
+      (:translate symbol-value)
       (:policy :fast-safe)
       (:args (symbol :scs (descriptor-reg constant immediate) :to (:result 1)))
       ;; TODO: use no temp if the symbol is known to be thread-local
@@ -338,8 +398,8 @@
       (:temporary (:sc unsigned-reg) temp)
       (:generator 9
         (inst mov :dword temp (tls-index-of object))
-        (inst mov :dword temp (thread-tls-ea temp))
-        (inst cmp :dword temp no-tls-value-marker-widetag)
+        (inst mov :qword temp (thread-tls-ea temp))
+        (inst cmp :qword temp no-tls-value-marker)
         (inst cmov :dword :e temp (symbol-value-slot-ea object))
         (inst cmp :byte temp unbound-marker-widetag))))
 
@@ -348,9 +408,9 @@
 #-sb-thread
 (progn
   (define-vop (symbol-value symbol-global-value)
-    (:translate symeval))
+    (:translate symbol-value))
   (define-vop (fast-symbol-value fast-symbol-global-value)
-    (:translate symeval))
+    (:translate symbol-value))
   (define-vop (set %set-symbol-global-value))
   (define-vop (boundp)
     (:translate boundp)
@@ -397,17 +457,6 @@
     GOOD
     (inst and res (lognot fixnum-tag-mask)))) ; redundant (but ok) if asm routine used
 
-(define-vop ()
-  (:policy :fast-safe)
-  (:translate sb-impl::install-hash-table-lock)
-  (:args (arg :scs (descriptor-reg)))
-  (:results (res :scs (descriptor-reg)))
-  (:vop-var vop)
-  (:generator 5
-    (inst push arg)
-    (invoke-asm-routine 'call 'sb-impl::install-hash-table-lock vop)
-    (inst pop res)))
-
 (eval-when (:compile-toplevel)
   ;; assumption: any object can be read 1 word past its base pointer
   (assert (= sb-vm:symbol-hash-slot 1)))
@@ -442,6 +491,24 @@
     (inst and base-ptr (lognot lowtag-mask))
     (inst mov res (ea n-word-bytes base-ptr)) ; 1 word beyond the header
     (inst and res (lognot fixnum-tag-mask))))
+
+(define-vop ()
+  (:args (symbol :scs (descriptor-reg)))
+  (:results (result :scs (unsigned-reg)))
+  (:result-types positive-fixnum)
+  (:translate sb-impl::symbol-package-id)
+  (:policy :fast-safe)
+  (:generator 2 ; ASSUMPTION: symbol-package-bits = 16
+   (inst mov :word result (object-slot-ea symbol symbol-name-slot (- other-pointer-lowtag 6)))
+   (inst movzx '(:word :dword) result result)))
+(define-vop ()
+  (:args (symbol :scs (descriptor-reg)))
+  (:results (result :scs (descriptor-reg) :from :load))
+  (:translate symbol-name)
+  (:policy :fast-safe)
+  (:generator 2
+   (inst mov result (1- (ash 1 sb-impl::symbol-name-bits)))
+   (inst and result (object-slot-ea symbol symbol-name-slot other-pointer-lowtag))))
 
 ;;;; fdefinition (FDEFN) objects
 
@@ -697,15 +764,23 @@
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg)))
   (:info offset)
+  (:vop-var vop)
+  ;; temp is wasted if we don't need a barrier, which we almost never do
+  (:temporary (:sc unsigned-reg) temp)
   (:generator 4
-    ;; TODO: gencgc does not need EMIT-GC-STORE-BARRIER here, but other other GC strategies might.
+    (let* ((value-tn (tn-ref-tn (tn-ref-across (vop-args vop))))
+           (prim-type (sb-c::tn-primitive-type value-tn))
+           (scs (and prim-type (sb-c::primitive-type-scs prim-type))))
+      (when (and (not (singleton-p scs))
+                 (member descriptor-reg-sc-number scs))
+        (emit-gc-store-barrier object nil temp (vop-nth-arg 1 vop) value)))
     (storew value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 (define-vop (closure-init-from-fp)
   (:args (object :scs (descriptor-reg)))
   (:info offset)
   (:generator 4
-    ;; TODO: gencgc does not need EMIT-GC-STORE-BARRIER here, but other other GC strategies might.
+    ;; RBP-TN looks like a fixnum (non-pointer) so no barrier
     (storew rbp-tn object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 ;;;; value cell hackery
@@ -823,6 +898,10 @@
 (define-full-compare-and-swap %raw-instance-cas/word instance
   instance-slots-offset instance-pointer-lowtag
   (unsigned-reg) unsigned-num %raw-instance-cas/word)
+(define-full-compare-and-swap %raw-instance-cas/signed-word instance
+  instance-slots-offset instance-pointer-lowtag
+  (signed-reg) signed-num %raw-instance-cas/signed-word)
+
 (define-vop ()
   (:translate %raw-instance-xchg/word)
   (:policy :fast-safe)
