@@ -17,30 +17,20 @@
 (defconstant sb-assem:+inst-alignment-bytes+ 1)
 
 (defconstant +backend-fasl-file-implementation+ :x86-64)
-(defconstant-eqx +fixup-kinds+ #(:absolute :relative :absolute64)
-  #'equalp)
+(defconstant-eqx +fixup-kinds+ #(:abs32 :rel32 :absolute) #'equalp)
 
 ;;; This size is supposed to indicate something about the actual granularity
 ;;; at which you can map memory.  We just hardwire it, but that may or may not
 ;;; be necessary any more.
 (defconstant +backend-page-bytes+ #+win32 65536 #-win32 32768)
 
-;;; The size in bytes of GENCGC cards, i.e. the granularity at which
-;;; writes to old generations are logged.
-;;; The size is a trade-off between efficiency of the allocator
-;;; and efficiency of scanning. Generally a card-marking GC will use cards
-;;; that are fairly small - The JVM used to use 512, I don't know if it still does.
-;;; I've heard of as small as 64 bytes being used in academic papers.
-;;; SBCL however has a problem with small sizes for two reasons:
-;;; (1) the size in which we claim memory in the slow-path allocator is exactly
-;;; the card size times a multiplier that does not work very well, and in fact
-;;; will work worse once I checkin a change to improve concurrency within the
-;;; slow path (2) heap scans can't begin at an arbitrary card because it might
-;;; the middle of a partly-boxed object. So we need to distinguish between
-;;; *strictly* boxed pages, and mixed-tagged/raw-word pages.
-;;; Alternatively, GC could try to skip over the cards at the start of a contiguous
-;;; block until it gets to the cards that are actully marked.
-(defconstant gencgc-card-bytes 16384)
+;;; The size in bytes of GENCGC pages. A page is the smallest amount of memory
+;;; that a thread can claim for a thread-local region, and also determines
+;;; the granularity at which we can find the start of a sequence of objects.
+(defconstant gencgc-page-bytes 32768)
+;;; The divisor relative to page-bytes which computes the granularity
+;;; at which writes to old generations are logged.
+(defconstant cards-per-page 32)
 ;;; The minimum size of new allocation regions.  While it doesn't
 ;;; currently make a lot of sense to have a card size lower than
 ;;; the alloc granularity, it will, once we are smarter about finding
@@ -114,7 +104,7 @@
 
 #-(or linux darwin)
 (!gencgc-space-setup #x20000000
-                     #-win32 :read-only-space-size #-win32 0
+                     :read-only-space-size 0
                      :dynamic-space-start #x1000000000
                      #+openbsd :dynamic-space-size #+openbsd #x1bcf0000)
 
@@ -192,7 +182,17 @@
   #'equalp)
 
 #+sb-simd-pack
-(defglobal *simd-pack-element-types* '(integer single-float double-float))
+(defglobal *simd-pack-element-types*
+    '(single-float
+      double-float
+      (unsigned-byte 8)
+      (unsigned-byte 16)
+      (unsigned-byte 32)
+      (unsigned-byte 64)
+      (signed-byte 8)
+      (signed-byte 16)
+      (signed-byte 32)
+      (signed-byte 64)))
 
 (defconstant undefined-fdefn-header
   ;; This constant is constructed as follows: Take the INT opcode
