@@ -231,12 +231,17 @@
 
 ;;;; other integer ranges
 
+(define-vop (simple-type-predicate)
+  (:args (value :scs (any-reg descriptor-reg control-stack)))
+  (:conditional)
+  (:args-var args)
+  (:policy :fast-safe))
+
 (define-vop (fixnump/unsigned-byte-64 simple-type-predicate)
   (:args (value :scs (unsigned-reg)))
   (:arg-types unsigned-num)
   (:translate fixnump)
   (:temporary (:sc unsigned-reg :from (:argument 0)) tmp)
-  (:info)
   (:conditional :z)
   (:generator 3
     (move tmp value)
@@ -245,7 +250,6 @@
 #-#.(cl:if (cl:= sb-vm:n-fixnum-tag-bits 1) '(:and) '(:or))
 (define-vop (fixnump/signed-byte-64 simple-type-predicate)
   (:args (value :scs (signed-reg)))
-  (:info)
   (:conditional :z)
   (:temporary (:sc unsigned-reg) temp)
   (:arg-types signed-num)
@@ -262,7 +266,6 @@
 #+#.(cl:if (cl:= sb-vm:n-fixnum-tag-bits 1) '(:and) '(:or))
 (define-vop (fixnump/signed-byte-64 simple-type-predicate)
   (:args (value :scs (signed-reg) :target temp))
-  (:info)
   (:conditional :no)
   (:temporary (:sc unsigned-reg :from (:argument 0)) temp)
   (:arg-types signed-num)
@@ -275,55 +278,117 @@
 ;;; A (SIGNED-BYTE 64) can be represented with either fixnum or a bignum with
 ;;; exactly one digit.
 
-(defmacro bignum-header-for-length (n)
-  (logior (ash n n-widetag-bits) bignum-widetag))
+(define-vop (pointerp)
+  (:args (value :scs (any-reg descriptor-reg)))
+  (:temporary (:sc unsigned-reg :from (:argument 0)) temp)
+  (:conditional :z)
+  (:policy :fast-safe)
+  (:translate pointerp)
+  (:generator 3
+    (if (location= temp value) (inst sub :dword value 3) (inst lea :dword temp (ea -3 value)))
+    (inst test :byte temp #b11)))
 
-(define-vop (signed-byte-64-p type-predicate)
+;; A fixnum or single-digit bignum satisfies signed-byte-64-p
+(define-vop (signed-byte-64-p pointerp)
   (:translate signed-byte-64-p)
-  (:generator 45
-    (multiple-value-bind (yep nope)
-        (if not-p
-            (values not-target target)
-            (values target not-target))
-      (case n-fixnum-tag-bits
-        (1 (%lea-for-lowtag-test temp value other-pointer-lowtag)
-           (inst test :byte temp fixnum-tag-mask) ; 0th bit = 1 => fixnum
-           (inst jmp :nz yep)
-           (inst test :byte temp lowtag-mask))
-        (t ;; we'll only examine 1 byte, but moving a :dword is preferable
-           ;; as it doesn't require the CPU to retain the other 7 bytes.
-           (inst mov :dword temp value)
-           (inst test :byte temp fixnum-tag-mask)
-           (inst jmp :e yep)
-           (inst and :byte temp lowtag-mask)
-           (inst cmp :byte temp other-pointer-lowtag)))
-      (inst jmp :ne nope)
-      (inst cmp :qword (ea (- other-pointer-lowtag) value) (bignum-header-for-length 1))
-      (inst jmp (if not-p :ne :e) target))
-    NOT-TARGET))
+  (:conditional :z)
+  (:args-var arg-ref)
+  (:generator 6
+    (when (types-equal-or-intersect (tn-ref-type arg-ref) (specifier-type 'fixnum))
+      (inst test :byte value fixnum-tag-mask)
+      (inst jmp :z out)) ; good
+    (let ((ea (cond ((fixnum-or-other-pointer-tn-ref-p arg-ref)
+                     (ea (- other-pointer-lowtag) value))
+                    (t
+                     (%lea-for-lowtag-test temp value other-pointer-lowtag :qword)
+                     (inst test :byte temp lowtag-mask)
+                     (inst jmp :nz out)
+                     (ea temp)))))
+      (inst cmp :qword ea (bignum-header-for-length 1)))
+    OUT))
+
+(define-vop (signed-byte-64-p/unsigned)
+  (:args (value :scs (unsigned-reg)))
+  (:arg-types unsigned-num)
+  (:conditional :ns)
+  (:policy :fast-safe)
+  (:translate signed-byte-64-p)
+  (:generator 5
+    (inst test value value)))
+
+(macrolet ((define (name src-size)
+             `(progn
+                (define-vop (,name)
+                  (:translate ,name)
+                  (:args (value :scs (any-reg descriptor-reg)))
+                  (:conditional :z)
+                  (:args-var arg-ref)
+                  (:policy :fast-safe)
+                  (:temporary (:sc unsigned-reg) temp temp2)
+                  (:generator 6
+                    ;; Optimistically assume that the argument is a fixnum.
+                    ;; If it is, then we're shifting out the tag bit, otherwise
+                    ;; putting random bits in the low byte, which is harmless.
+                    (move temp value)
+                    (inst sar temp n-fixnum-tag-bits)
+                    (inst movsx '(,src-size :qword) temp2 temp)
+                    (inst cmp temp2 temp)
+                    ;; If the input wasn't known to be a fixnum, and it passed
+                    ;; the sign-extension test, then check FIXNUMP now.
+                    (unless (csubtypep (tn-ref-type arg-ref) (specifier-type 'fixnum))
+                      (inst jmp :nz OUT) ; did NOT pass the sign-extension test
+                      (inst test :byte value fixnum-tag-mask))
+                    OUT))
+                (define-vop (,(symbolicate name "/SIGNED"))
+                  (:translate ,name)
+                  (:args (value :scs (signed-reg)))
+                  (:arg-types signed-num)
+                  (:conditional :z)
+                  (:policy :fast-safe)
+                  (:temporary (:sc unsigned-reg) temp)
+                  (:generator 2
+                    (inst movsx '(,src-size :qword) temp value)
+                    (inst cmp temp value))))))
+  (define signed-byte-8-p :byte)
+  (define signed-byte-16-p :word)
+  (define signed-byte-32-p :dword))
+
+;;; Sign bit and fixnum tag bit.
+(defconstant non-negative-fixnum-mask-constant
+  #x8000000000000001)
+(defconstant non-negative-fixnum-mask-constant-wired-address
+  (+ static-space-start (* 12 n-word-bytes)))
+;; the preceding constant is embedded in an array,
+;; the header of which must not overlap the static alloc regions
+#-sb-thread
+(aver (>= (- non-negative-fixnum-mask-constant-wired-address (* 2 n-word-bytes))
+          (+ (max boxed-region cons-region mixed-region) (* 3 n-word-bytes))))
 
 ;;; An (unsigned-byte 64) can be represented with either a positive
 ;;; fixnum, a bignum with exactly one positive digit, or a bignum with
 ;;; exactly two digits and the second digit all zeros.
 (define-vop (unsigned-byte-64-p type-predicate)
   (:translate unsigned-byte-64-p)
-  (:generator 45
+  (:generator 10
     (let ((not-target (gen-label))
           (single-word (gen-label))
-          (fixnum (gen-label)))
+          (fixnum-p (types-equal-or-intersect (tn-ref-type args) (specifier-type 'fixnum))))
       (multiple-value-bind (yep nope)
           (if not-p
               (values not-target target)
               (values target not-target))
-        ;; Is it a fixnum?
-        (inst mov temp value)
-        (inst test :byte temp fixnum-tag-mask)
-        (inst jmp :e fixnum)
-
-        ;; If not, is it an other pointer?
-        (inst and :byte temp lowtag-mask)
-        (inst cmp :byte temp other-pointer-lowtag)
-        (inst jmp :ne nope)
+        (when fixnum-p
+          ;; Is it a fixnum with the sign bit clear?
+          (inst test (ea non-negative-fixnum-mask-constant-wired-address) value)
+          (inst jmp :z yep))
+        (cond ((fixnum-or-other-pointer-tn-ref-p args)
+               (when fixnum-p
+                 (inst test :byte value fixnum-tag-mask)
+                 (inst jmp :z nope)))
+              (t
+               (%lea-for-lowtag-test temp value other-pointer-lowtag)
+               (inst test :byte temp lowtag-mask)
+               (inst jmp :ne nope)))
         ;; Get the header.
         (loadw temp value 0 other-pointer-lowtag)
         ;; Is it one?
@@ -342,9 +407,7 @@
         (emit-label single-word)
         ;; Get the single digit.
         (loadw temp value bignum-digits-offset other-pointer-lowtag)
-
         ;; positive implies (unsigned-byte 64).
-        (emit-label fixnum)
         (inst test temp temp)
         (inst jmp (if not-p :s :ns) target)
 
@@ -366,8 +429,13 @@
   (:generator 4
      (let* ((fixnum-hi (if (sc-is value unsigned-reg signed-reg)
                            hi
-                           (fixnumize hi))))
-       (inst test value (constantize (lognot fixnum-hi))))))
+                           (fixnumize hi)))
+            (mask (lognot fixnum-hi))
+            (constant
+             (if (= (ldb (byte 64 0) mask) non-negative-fixnum-mask-constant)
+                 (ea non-negative-fixnum-mask-constant-wired-address)
+                 (constantize mask))))
+       (inst test value constant))))
 
 (define-vop (test-fixnum-mod-tagged-unsigned)
   (:args (value :scs (any-reg unsigned-reg signed-reg)))
@@ -404,23 +472,12 @@
 (macrolet ((define (name widetag)
              `(define-vop (,name simple-type-predicate)
                 (:translate ,name)
-                (:info)
                 (:conditional :z)
                 (:generator 1 (inst cmp :byte value ,widetag)))))
   (define single-float-p single-float-widetag)
   (define characterp character-widetag)
   (define unbound-marker-p unbound-marker-widetag))
 
-(define-vop (pointerp)
-  (:args (value :scs (any-reg descriptor-reg)))
-  (:temporary (:sc unsigned-reg :from (:argument 0)) temp)
-  (:conditional :z)
-  (:policy :fast-safe)
-  (:translate pointerp)
-  (:generator 3
-    (if (location= temp value) (inst sub :dword value 3) (inst lea :dword temp (ea -3 value)))
-    (inst test :byte temp #b11)))
-
 ;;; FUNCTIONP, LISTP, %INSTANCEP, %OTHER-POINTER-P produce a flag result
 (macrolet ((define (name lowtag)
              `(define-vop (,name pointerp)
@@ -435,10 +492,11 @@
   (define %instancep instance-pointer-lowtag)
   (define %other-pointer-p other-pointer-lowtag))
 
+;;; Function subtypes produce a flag result
 (macrolet ((define (name widetag)
              `(define-vop (,name type-predicate)
                 (:translate ,name)
-                (:info)
+                (:info) ; nullify the info
                 (:conditional :z)
                 (:generator 4
                   (inst lea temp (ea (- fun-pointer-lowtag) value))
@@ -449,12 +507,81 @@
   (define closurep closure-widetag)
   (define simple-fun-p simple-fun-widetag)
   (define funcallable-instance-p funcallable-instance-widetag))
+;;; Various OTHER-POINTER objects produce a flag result.
+;;; The parens around widetag are from copy&paste of generic/type-vops
+(macrolet ((define (name (widetag))
+             `(define-vop (,name type-predicate)
+                (:translate ,name)
+                (:info) ; nullify the info
+                (:conditional :z)
+                (:generator 4
+                  (test-other-ptr value args ,widetag temp out)
+                  out))))
+  (define bignump (bignum-widetag))
+  (define ratiop (ratio-widetag))
+  (define complex-rational-p (complex-widetag))
+  (define complex-single-float-p (complex-single-float-widetag))
+  (define complex-double-float-p (complex-double-float-widetag))
+  (define double-float-p (double-float-widetag))
+  (define system-area-pointer-p (sap-widetag))
+  (define weak-pointer-p (weak-pointer-widetag))
+  (define code-component-p (code-header-widetag))
+  (define fdefn-p (fdefn-widetag))
+  (define simple-array-header-p (simple-array-widetag))
+  (define complex-vector-p (complex-vector-widetag)))
+
+(macrolet ((fail-if-not-otherptr ()
+             `(cond ((other-pointer-tn-ref-p value-tn-ref)
+                     (inst mov :byte temp (ea (- other-pointer-lowtag) value)))
+                    (t
+                     (inst lea temp (ea (- other-pointer-lowtag) value))
+                     (inst test :byte temp lowtag-mask)
+                     ;; TEST clears the Carry, so if this jump occurs,
+                     ;; it returns the correct answer for the vops that return
+                     ;; their result in Z or C.
+                     (inst jmp :nz out)
+                     (inst mov :byte temp (ea temp))))))
+  #+sb-unicode
+  (macrolet ((define (name (simple nonsimple))
+               (aver (= (logior 8 (symbol-value simple)) (symbol-value nonsimple)))
+               `(define-vop (,name type-predicate)
+                  (:translate ,name)
+                  (:info) ; nullify the info
+                  (:conditional :z)
+                  (:args-var value-tn-ref)
+                  (:generator 4
+                    (fail-if-not-otherptr)
+                    (inst or :byte temp 8)
+                    (inst cmp :byte temp ,nonsimple)
+                    out))))
+    (define base-string-p (simple-base-string-widetag complex-base-string-widetag))
+    (define character-string-p
+        (simple-character-string-widetag complex-character-string-widetag)))
+  (macrolet ((define (name widetags)
+               (let* ((widetags (symbol-value widetags))
+                      (min (reduce #'min widetags))
+                      (max (reduce #'max widetags)))
+                 `(define-vop (,name type-predicate)
+                    (:translate ,name)
+                    (:info)
+                    (:conditional :c) ; Carry flag = "below" (unsigned)
+                    (:args-var value-tn-ref)
+                    (:generator 4
+                      (fail-if-not-otherptr)
+                      (inst sub :byte temp ,min)
+                      (inst cmp :byte temp ,(1+ (- max min)))
+                      OUT)))))
+    (define simple-rank-1-array-*-p +simple-rank-1-array-widetags+)
+    (define vectorp +vector-widetags+)
+    (define simple-array-p +simple-array-widetags+)
+    #+sb-unicode (define stringp +string-widetags+)))
 
 ;;;; list/symbol types
 ;;;
 ;;; symbolp (or symbol (eq nil))
 ;;; consp (and list (not (eq nil)))
 
+;;; Test whether ARG is an other-pointer to WIDETAG, setting the Z flag if so
 (defun test-other-ptr (arg arg-ref widetag temp label)
   (inst cmp :byte
         (cond ((other-pointer-tn-ref-p arg-ref)
@@ -471,16 +598,53 @@
   (:info)
   (:conditional :z)
   (:generator 5
-    ;; SYMBOLP would have been IR1-transformed to NON-NULL-SYMBOL-P when possible
+    ;; If the VALUE were known to be an OTHER-POINTER, the the IR1 lvar type
+    ;; can't intersect NULL. So the IR1 should have been transformed to NON-NULL-SYMBOL-P.
+    ;; Hence this must *not* be known to be an OTHER-POINTER.
+    (aver (not (other-pointer-tn-ref-p args)))
+    (%lea-for-lowtag-test temp value other-pointer-lowtag :qword)
+    (inst test :byte temp lowtag-mask)
+    (inst jmp :e compare-widetag)
     (inst cmp value nil-value)
-    (inst jmp :e out)
-    (test-other-ptr value args symbol-widetag temp out)
+    (inst jmp out)
+    compare-widetag
+    (inst cmp :byte (ea temp) symbol-widetag)
     out))
 
 (define-vop (non-null-symbol-p symbolp)
   (:translate non-null-symbol-p)
   (:generator 3 (test-other-ptr value args symbol-widetag temp out) out))
 
+;;; It would be far better if we could recognize the IR1 for
+;;;  (AND (CONSP X) (EQ (CAR X) 'FOO))
+;;; rather than treating (TYPEP X '(CONS (EQL FOO))) as a special case,
+;;; but hey at least this provides the IR2 support for it.
+(define-vop (car-eq-if-listp)
+  (:args (value :scs (descriptor-reg))
+         (obj :scs (immediate any-reg descriptor-reg)))
+  (:temporary (:sc unsigned-reg) temp)
+  (:conditional :z)
+  (:policy :fast-safe)
+  (:translate car-eq-if-listp)
+  (:generator 3
+    (inst lea temp (ea (- list-pointer-lowtag) value))
+    (inst test :byte temp lowtag-mask)
+    (inst jmp :nz out)
+    (inst cmp :qword (ea temp) (encode-value-if-immediate obj))
+    out))
+
+(eval-when (:compile-toplevel) (aver (= sb-impl::package-id-bits 16)))
+(define-vop (keywordp symbolp)
+  (:translate keywordp)
+  (:generator 3
+    (inst lea temp (ea (- other-pointer-lowtag) value))
+    (inst test :byte temp lowtag-mask)
+    (inst jmp :ne out)
+    (inst cmp :byte (ea temp) symbol-widetag)
+    (inst jmp :ne out)
+    (inst cmp :word (ea (+ (ash symbol-name-slot word-shift) 6) temp) sb-impl::+package-id-keyword+)
+    out))
+
 (define-vop (consp type-predicate)
   (:translate consp)
   (:generator 8
@@ -519,8 +683,10 @@
           (value :scs (any-reg descriptor-reg)))
    (:vop-var vop)
    (:temporary (:sc unsigned-reg) temp)
+   (:node-var node)
    (:generator 1
-     (emit-gc-store-barrier object nil temp (vop-nth-arg 1 vop) value)
+     (unless (sb-c::set-slot-old-p node)
+       (emit-gc-store-barrier object nil temp (vop-nth-arg 1 vop) value))
      (inst mov :dword (ea (- 4 instance-pointer-lowtag) object) value)))
  (define-vop (%fun-layout %instance-layout)
    (:translate %fun-layout)
@@ -528,7 +694,8 @@
  (define-vop (%set-fun-layout %set-instance-layout)
    (:translate %set-fun-layout)
    (:generator 1
-     (emit-gc-store-barrier object nil temp (vop-nth-arg 1 vop) value)
+     (unless (sb-c::set-slot-old-p node)
+       (emit-gc-store-barrier object nil temp (vop-nth-arg 1 vop) value))
      (inst mov :dword (ea (- 4 fun-pointer-lowtag) object) value)))
  (define-vop ()
   (:translate sb-c::layout-eq)
@@ -577,7 +744,6 @@
   (:translate fixnump)
   (:args-var arg-ref)
   (:args (value :scs (any-reg descriptor-reg) :load-if (tn-ref-memory-access arg-ref)))
-  (:info)
   (:conditional :z)
   ;; the compiler is very sensitive to this cost here as regards boxing. DON'T TOUCH !!!
   (:generator 3
@@ -637,3 +803,85 @@
               (sb-c::delete-vop prev)
               (sb-c::delete-vop vop)
               new)))))))
+
+(define-vop (>-integer-fixnum)
+  (:translate >)
+  (:args (integer :scs (descriptor-reg))
+         (fixnum :scs (immediate any-reg)))
+  (:arg-types (:or integer bignum) tagged-num)
+  (:temporary (:sc unsigned-reg) temp)
+  (:conditional)
+  (:info target not-p)
+  (:args-var args)
+  (:policy :fast-safe)
+  (:variant-vars comparison)
+  (:variant :g)
+  (:generator 8
+    (unless (sc-is (tn-ref-tn args) descriptor-reg control-stack)
+      (setf args (tn-ref-across args)))
+    (let* ((integer-p (csubtypep (tn-ref-type args) (specifier-type 'integer)))
+           (fixnum (if (sc-is fixnum immediate)
+                       (let* ((value (fixnumize (tn-value fixnum))))
+                         (cond ((plausible-signed-imm32-operand-p value)
+                                value)
+                               (t
+                                (inst mov temp value)
+                                temp)))
+                       fixnum)))
+      (multiple-value-bind (yep nope)
+          (if not-p
+              (values not-target target)
+              (values target not-target))
+        (assemble ()
+          (when (types-equal-or-intersect (tn-ref-type args) (specifier-type 'fixnum))
+            (generate-fixnum-test integer)
+            (inst jmp :nz BIGNUM)
+            (if (eql fixnum 0)
+                (inst test integer integer)
+                (inst cmp integer fixnum))
+            (inst jmp comparison yep)
+            (inst jmp nope))
+          bignum
+          (unless (fixnum-or-other-pointer-tn-ref-p args)
+            (test-type integer temp nope t (other-pointer-lowtag)))
+          (loadw temp integer 0 other-pointer-lowtag)
+          (unless integer-p
+            (inst cmp :byte temp bignum-widetag)
+            (inst jmp :ne nope))
+          #.(assert (= (integer-length bignum-widetag) 5))
+          (inst shr temp 5)
+          (inst cmp :qword (ea (- other-pointer-lowtag) integer temp) 0)
+          (inst jmp (case comparison
+                      ((:l :le) (if not-p :ge :l))
+                      (t (if not-p :l :ge)))
+                target))))
+    not-target))
+
+(define-vop (<-integer-fixnum >-integer-fixnum)
+  (:translate <)
+  (:variant :l))
+
+(define-vop (>-fixnum-integer >-integer-fixnum)
+  (:translate >)
+  (:args (fixnum :scs (immediate any-reg))
+         (integer :scs (descriptor-reg)))
+  (:arg-types tagged-num (:or integer bignum))
+  (:variant :l))
+
+(define-vop (<-fixnum-integer >-fixnum-integer)
+  (:translate <)
+  (:variant :g))
+
+;;; For integerp+cmp
+(define-vop (<=-integer-fixnum >-integer-fixnum)
+  (:translate)
+  (:variant :le))
+(define-vop (>=-integer-fixnum <-integer-fixnum)
+  (:translate)
+  (:variant :ge))
+(define-vop (<=-fixnum-integer >-fixnum-integer)
+  (:translate)
+  (:variant :ge))
+(define-vop (>=-fixnum-integer <-fixnum-integer)
+  (:translate)
+  (:variant :le))
