@@ -174,7 +174,8 @@
   #-win32
   (defregset    *c-call-register-arg-offsets* rdi rsi rdx rcx r8 r9)
   #+win32
-  (defregset    *c-call-register-arg-offsets* rcx rdx r8 r9))
+  (defregset    *c-call-register-arg-offsets* rcx rdx r8 r9)
+  (defregset *descriptor-args* rdx rdi rsi rbx rcx r8 r9 r10 r14 r15))
 
 ;;;; SB definitions
 
@@ -485,8 +486,9 @@
                ;; is used to determine whether it's immediate.
                #+(and (not sb-xc-host) immobile-space (not immobile-symbols))
                (or (logbitp +initial-core-symbol-bit+ (get-header-data value))
-                   (and (sb-c::core-object-p sb-c::*compile-object*)
-                        (immobile-space-obj-p value)))
+                   (locally (declare (notinline sb-c::producing-fasl-file))
+                     (and (not (sb-c::producing-fasl-file))
+                          (immobile-space-obj-p value))))
 
                (static-symbol-p value))
        immediate-sc-number))
