@@ -143,20 +143,59 @@
 
 ;;; Reset CTOR to use a default function that will compute an
 ;;; optimized constructor function when called.
+;;;
+;;; Note that lp#1951341 saw a very rare failure which suggested
+;;; a memory ordering issue. Indeed, here is what happened:
+;;;  - thread A allocates a CTOR
+;;;    CTOR    ->   +-------------------------------+
+;;;                 | word 0: funinstance header    |
+;;;                 | word 1: trampline             |
+;;;                 | word 2: layout-of-CTOR        |
+;;;                 | word 3: funinstance function  |
+;;;                 | ... more slots ...            |
+;;;                 +-------------------------------+
+;;;
+;;;  - thread A creates a lambda that captures CTOR
+;;;    CLOSURE ->   +------------------------------+
+;;;                 | word 0: closure header       |
+;;;                 | word 1: underlying function  |
+;;;                 | word 2: CTOR                 |
+;;;                 +------------------------------+
+;;;
+;;; - thread A stores CLOSURE into word 3 of CTOR
+;;;   by way of (SETF (%FUNCALLABLE-INSTANCE-FUN CTOR) ...)
+;;;
+;;; - thread B gets ahold of CTOR - how? I'm not clear
+;;;   on the protocol, but I guess it was already installed
+;;;   as the class's constructor
+;;;
+;;; - thread B reads word 3 out of the CTOR and calls
+;;;   the CLOSURE (successfully entering into its instructions)
+;;;
+;;; - CLOSURE's code loads CTOR from itself and calls
+;;;   INSTALL-OPTIMIZED-CONSTRUCTOR with an argument of 0
+;;;   because B did not observe a store into word 2
+;;;   of CLOSURE
+;;;
+;;; The fix is to use properly paired barriers.
 (defun install-initial-constructor (ctor &optional force-p)
   (when (or force-p (ctor-class ctor))
     (setf (ctor-class ctor) nil
           (ctor-state ctor) 'initial)
-    (setf (%funcallable-instance-fun ctor)
-          (ecase (ctor-type ctor)
-            (ctor
-             (lambda (&rest args)
-               (install-optimized-constructor ctor)
-               (apply ctor args)))
-            (allocator
-             (lambda ()
-               (install-optimized-allocator ctor)
-               (funcall ctor)))))))
+    (let ((closure
+           (ecase (ctor-type ctor)
+             (ctor
+              (lambda (&rest args)
+                (sb-thread:barrier (:read))
+                (install-optimized-constructor ctor)
+                (apply ctor args)))
+             (allocator
+              (lambda ()
+                (sb-thread:barrier (:read))
+                (install-optimized-allocator ctor)
+                (funcall ctor))))))
+      (sb-thread:barrier (:write))
+      (setf (%funcallable-instance-fun ctor) closure))))
 
 (defun make-ctor-function-name (class-name initargs safe-code-p)
   (labels ((arg-name (x)
@@ -552,12 +591,9 @@
       (multiple-value-bind (form locations names optimizedp)
           (constructor-function-form ctor)
         (setf (%funcallable-instance-fun ctor)
-              (apply
-               (let ((*compiling-optimized-constructor* t))
-                 (handler-bind ((compiler-note #'muffle-warning))
-                   (compile nil `(lambda ,names (declare #.*optimize-speed*)
-                                   ,form))))
-               locations)
+              (apply (let ((*compiling-optimized-constructor* t))
+                       (pcl-compile `(lambda ,names ,form) t))
+                     locations)
               (ctor-state ctor) (if optimizedp 'optimized 'fallback))))))
 
 (defun install-optimized-allocator (ctor)
@@ -580,9 +616,7 @@
       (multiple-value-bind (form optimizedp)
           (allocator-function-form ctor)
         (setf (%funcallable-instance-fun ctor)
-              (let ((*compiling-optimized-constructor* t))
-                (handler-bind ((compiler-note #'muffle-warning))
-                  (compile nil form)))
+              (let ((*compiling-optimized-constructor* t)) (pcl-compile form t))
               (ctor-state ctor) (if optimizedp 'optimized 'fallback))))))
 
 (defun allocator-function-form (ctor)
@@ -782,12 +816,11 @@
          (wrapper (class-wrapper class)))
     (etypecase class
       (standard-class
-        `(let ((.instance. (%make-instance (1+ sb-vm:instance-data-start)))
+        `(let ((.instance. (%new-instance ,wrapper (1+ sb-vm:instance-data-start)))
                (.slots. (make-array
                          ,(wrapper-length wrapper)
                          ,@(when early-unbound-markers-p
                                  '(:initial-element +slot-unbound+)))))
-           (setf (%instance-wrapper .instance.) ,wrapper)
            (%instance-set .instance. sb-vm:instance-data-start .slots.)
            ,body
            .instance.))
