@@ -96,6 +96,8 @@ static void* get_free_page(boolean prezero) {
     char* mem = page_address(free_page);
 #ifdef LISP_FEATURE_DARWIN_JIT
     prezero = 1; // Might need to alter MMU-based protection
+#elif defined LISP_FEATURE_WIN32
+    os_commit_memory(page_address(free_page), GENCGC_PAGE_BYTES);
 #endif
     if (prezero) zeroize_pages_if_needed(free_page, free_page, 0);
     return mem;
@@ -208,7 +210,6 @@ static inline int pointer_survived_gc_yet(lispobj pointer)
     return (header & MARK_BIT) != 0;
 }
 
-extern page_index_t contiguous_block_final_page(page_index_t);
 void dump_marked_objects() {
     fprintf(stderr, "Marked objects:\n");
     page_index_t first = 0;
@@ -420,7 +421,7 @@ static void trace_object(lispobj* where)
         return;
 #endif
     case FDEFN_WIDETAG:
-        gc_mark_obj(fdefn_callee_lispobj((struct fdefn*)where));
+        gc_mark_obj(decode_fdefn_rawfun((struct fdefn*)where));
         scan_to = 3;
         break;
     case WEAK_POINTER_WIDETAG:
@@ -714,7 +715,7 @@ void execute_full_sweep_phase()
     for (page = free_page; page < page_table_pages; ++page) {
         gc_assert((page_table[page].type & PAGE_TYPE_MASK) == PAGE_TYPE_UNBOXED);
         gc_assert(!page_bytes_used(page));
-        set_page_needs_zerofill(page);
+        set_page_need_to_zero(page, 1);
         page_table[page].type = FREE_PAGE_FLAG;
       }
 }
