@@ -40,6 +40,9 @@ extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
 
 #include "align.h"
 
+// Distinguish penultimate GC (iteration 1) from ultimate GC (iteration 2) in save-lisp
+extern int save_lisp_gc_iteration;
+
 // Offset from an fdefn raw address to the underlying simple-fun,
 // if and only if it points to a simple-fun.
 // For those of us who are too memory-impaired to know how to use the value:
@@ -51,9 +54,9 @@ extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
 #define FUN_RAW_ADDR_OFFSET (offsetof(struct simple_fun, insts) - FUN_POINTER_LOWTAG)
 #endif
 
-// For x86[-64], a simple-fun or closure's "self" slot is a fixum
+// For x86[-64], arm64, a simple-fun or closure's "self" slot is a fixum
 // On other backends, it is a lisp ointer.
-#if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
+#if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64) || defined(LISP_FEATURE_ARM64)
 #define FUN_SELF_FIXNUM_TAGGED 1
 #else
 #define FUN_SELF_FIXNUM_TAGGED 0
@@ -86,48 +89,46 @@ extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
  */
 #define vector_flagp(header, val) ((int)header & (flag_##val << ARRAY_FLAGS_POSITION))
 #define vector_flags_zerop(header) ((int)(header) & 0x07 << ARRAY_FLAGS_POSITION) == 0
-// True if flags are zero, also testing the widetag at the same time.
-#define ordinary_simple_vector_p(header) \
-    ((int)(header) & (0x07 << ARRAY_FLAGS_POSITION | 0xffff >> (16-ARRAY_FLAGS_POSITION))) \
-    == SIMPLE_VECTOR_WIDETAG
 // Return true if vector is a weak vector that is not a hash-table <k,v> vector.
-#define vector_is_weak_not_hashing_p(header) \
-  ((int)(header) & ((flag_VectorWeak|flag_VectorHashing) << ARRAY_FLAGS_POSITION)) == \
-    (flag_VectorWeak << ARRAY_FLAGS_POSITION)
+static inline int vector_is_weak_not_hashing_p(unsigned int header) {
+    return (header & ((flag_VectorWeak|flag_VectorHashing) << ARRAY_FLAGS_POSITION)) ==
+      flag_VectorWeak << ARRAY_FLAGS_POSITION;
+}
 
-// Mask out the fullcgc mark bit when asserting header validity
+// This bit can be anything that doesn't conflict with the fullcgc mark bit or a bit
+// seen by lisp. Byte index 0 is the widetag, byte indices 1 and 2 are for the array-rank
+// and vector-flags, depending on how src/compiler/generic/early-objdef assigns them.
+#define WEAK_VECTOR_VISITED_BIT (1<<30)
+
+// Assert that the 'v' is a weak (not hashing) simple-vector and was visited,
+// and then clear the visited bit.
 #define UNSET_WEAK_VECTOR_VISITED(v) \
-  gc_assert((v->header & (0xff << ARRAY_FLAGS_POSITION | 0xff)) ==     \
-    (((flag_VectorWeakVisited|flag_VectorWeak) << ARRAY_FLAGS_POSITION) \
-     | SIMPLE_VECTOR_WIDETAG)); \
-  v->header ^= flag_VectorWeakVisited << ARRAY_FLAGS_POSITION
-
-/* values for the *_alloc_* parameters, also see the commentary for
- * struct page in gencgc-internal.h. These constants are used in gc-common,
- * so they can't easily be made gencgc-only */
-#define FREE_PAGE_FLAG        0
-#define PAGE_TYPE_MASK        7 // mask out the 'single-object flag'
-/* Note: lisp's CLOSE-CURRENT-GC-REGION expects BOXED_PAGE_FLAG = 1.
- * (probably should do a foreign call rather than kludge it) */
-#define BOXED_PAGE_FLAG       1
-#define UNBOXED_PAGE_FLAG     2
-/* CONS_PAGE_FLAG doesn't get stored in the page table, though I am considering
- * doing that. If conses went on segregated pages, then testing for a valid
- * conservative root on a cons page is as simple as seeing whether the address
- * is correctly aligned and lowtagged.
- * Also, we could reserve bytes at the end of each page to act as a mark bitmap
- * which is useful since conses are headerless objects, and one GC strategy
- * demands mark bitmaps which are currently placed in a side table.
- * That would unfortunately complicate the task of allocating a huge list,
- * because hitting the line of demarcation between conses and the mark bits would
- * require chaining the final cons to another page of conses and so on. */
-#define CONS_PAGE_FLAG        4
-#define OPEN_REGION_PAGE_FLAG 8
-#define CODE_PAGE_TYPE        (BOXED_PAGE_FLAG|UNBOXED_PAGE_FLAG)
+  gc_assert((v->header & (WEAK_VECTOR_VISITED_BIT | 0xff << ARRAY_FLAGS_POSITION | 0xff)) == \
+    (WEAK_VECTOR_VISITED_BIT | flag_VectorWeak << ARRAY_FLAGS_POSITION | SIMPLE_VECTOR_WIDETAG)); \
+  v->header ^= WEAK_VECTOR_VISITED_BIT
 
 extern sword_t (*sizetab[256])(lispobj *where);
-#define OBJECT_SIZE(header,where) \
-  (is_header(header)?sizetab[header_widetag(header)](where):CONS_SIZE)
+typedef sword_t (*sizerfn)(lispobj*);
+static inline sword_t object_size(lispobj* where) {
+    sizerfn f = sizetab[widetag_of(where)];
+    return f ? f(where) : CONS_SIZE;
+}
+// These three variants are potentially more efficient -
+//  (1) if the widetag was loaded, avoids one memory read
+//  (2) if you know the object isn't a cons, use headerobj_size
+//  (3) both of the above pertain.
+// Cases 1 and 3 exist only because C doesn't have optional args.
+// These might be premature optimizations, I really don't know.
+static inline sword_t object_size2(lispobj* where, unsigned int header) {
+    sizerfn f = sizetab[header & WIDETAG_MASK];
+    return f ? f(where) : CONS_SIZE;
+}
+static inline sword_t headerobj_size(lispobj* where) {
+    return sizetab[widetag_of(where)](where);
+}
+static inline sword_t headerobj_size2(lispobj* where, unsigned int header) {
+    return sizetab[header & WIDETAG_MASK](where);
+}
 
 lispobj *gc_search_space3(void *pointer, lispobj *start, void *limit);
 static inline lispobj *gc_search_space(lispobj *start, void *pointer) {
@@ -136,8 +137,6 @@ static inline lispobj *gc_search_space(lispobj *start, void *pointer) {
                             (void*)(1+((lispobj)pointer | LOWTAG_MASK)));
 }
 
-struct vector *symbol_name(lispobj*);
-
 extern void scrub_control_stack(void);
 extern void scrub_thread_control_stack(struct thread *);
 
@@ -162,6 +161,7 @@ instance_scan(void (*proc)(lispobj*, sword_t, uword_t),
 
 extern int simple_fun_index(struct code*, struct simple_fun*);
 
-extern lispobj fdefn_callee_lispobj(struct fdefn *fdefn);
-
+extern lispobj decode_fdefn_rawfun(struct fdefn *fdefn);
+extern void gc_close_thread_regions(struct thread*);
+extern void gc_close_collector_regions(int);
 #endif /* _GC_INTERNAL_H_ */
