@@ -45,6 +45,9 @@ extern int foreign_function_call_active;
 extern os_vm_size_t dynamic_space_size;
 extern os_vm_size_t thread_control_stack_size;
 
+#ifndef LISP_FEATURE_DARWIN_JIT
+extern uword_t READ_ONLY_SPACE_START, READ_ONLY_SPACE_END;
+#endif
 #ifdef LISP_FEATURE_CHENEYGC
 extern uword_t DYNAMIC_0_SPACE_START, DYNAMIC_1_SPACE_START;
 #else
@@ -87,6 +90,10 @@ extern lispobj *current_binding_stack_pointer;
 extern lispobj *read_only_space_free_pointer;
 extern lispobj *static_space_free_pointer;
 
+static inline boolean readonly_space_p(lispobj ptr) {
+    return ptr >= READ_ONLY_SPACE_START && (lispobj*)ptr < read_only_space_free_pointer;
+}
+
 #ifdef LISP_FEATURE_DARWIN_JIT
 lispobj *static_code_space_free_pointer;
 #endif
