@@ -59,15 +59,6 @@ void os_invalidate(os_vm_address_t addr, os_vm_size_t len)
         perror("munmap");
 }
 
-
-void
-os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
-{
-    if(mprotect((void*)address, length, prot) == -1) {
-        perror("mprotect");
-    }
-}
-
 #if defined LISP_FEATURE_GENCGC
 
 void
@@ -75,7 +66,7 @@ sigsegv_handler(int signal, siginfo_t *info, os_context_t *context)
 {
     void* fault_addr = (void*)info->si_addr;
 
-    if (gencgc_handle_wp_violation(fault_addr)) return;
+    if (gencgc_handle_wp_violation(context, fault_addr)) return;
 
     if (!handle_guard_page_triggered(context, fault_addr))
             lisp_memory_fault_error(context, fault_addr);
