@@ -50,6 +50,7 @@
 #include "pseudo-atomic.h"
 #include "interrupt.h"
 #include "lispregs.h"
+#include "atomiclog.inc"
 
 #ifdef LISP_FEATURE_SB_THREAD
 
@@ -279,7 +280,7 @@ void* read_current_thread() {
 extern pthread_key_t foreign_thread_ever_lispified;
 #endif
 
-#if !defined COLLECT_GC_STATS && \
+#if !defined COLLECT_GC_STATS && !defined STANDALONE_LDB && \
   defined LISP_FEATURE_LINUX && defined LISP_FEATURE_SB_THREAD && defined LISP_FEATURE_64_BIT
 #define COLLECT_GC_STATS
 #endif
@@ -307,13 +308,29 @@ void reset_gc_stats() { // after sb-posix:fork
 }
 #endif
 
+#ifdef ATOMIC_LOGGING
+#define THREAD_NAME_MAP_MAX 20 /* KLUDGE */
+struct {
+  pthread_t thread;
+  char *name; // strdup'ed
+} thread_name_map[THREAD_NAME_MAP_MAX];
+int thread_name_map_count;
+
+char* thread_name_from_pthread(pthread_t pointer){
+    int i;
+    for(i=0; i<thread_name_map_count; ++i)
+        if (thread_name_map[i].thread == pointer) return thread_name_map[i].name;
+    return 0;
+}
+#endif
+
 void create_main_lisp_thread(lispobj function) {
 #ifdef LISP_FEATURE_WIN32
     InitializeCriticalSection(&all_threads_lock);
     InitializeCriticalSection(&recyclebin_lock);
     InitializeCriticalSection(&in_gc_lock);
 #endif
-    struct thread *th = alloc_thread_struct(0, NO_TLS_VALUE_MARKER_WIDETAG);
+    struct thread *th = alloc_thread_struct(0, NO_TLS_VALUE_MARKER);
     if (!th || arch_os_thread_init(th)==0 || !init_shared_attr_object())
         lose("can't create initial thread");
     th->state_word.sprof_enable = 1;
@@ -552,6 +569,14 @@ void* new_thread_trampoline(void* arg)
 #endif
     }
 
+#ifdef ATOMIC_LOGGING
+      char* string = strdup((char*)VECTOR(name)->data);
+      int index = __sync_fetch_and_add(&thread_name_map_count, 1);
+      gc_assert(index < THREAD_NAME_MAP_MAX);
+      thread_name_map[index].thread = pthread_self();
+      thread_name_map[index].name = string;
+#endif
+
     struct vector* startup_info = VECTOR(lispthread->startup_info); // 'lispthread' is pinned
     gc_assert(header_widetag(startup_info->header) == SIMPLE_VECTOR_WIDETAG);
     lispobj startfun = startup_info->data[0]; // 'startup_info' is pinned
@@ -596,7 +621,7 @@ void* new_thread_trampoline(void* arg)
     init_thread_data scribble;
 
     lispobj function = th->no_tls_value_marker;
-    th->no_tls_value_marker = NO_TLS_VALUE_MARKER_WIDETAG;
+    th->no_tls_value_marker = NO_TLS_VALUE_MARKER;
     init_new_thread(th, &scribble,
                     GUARD_CONTROL_STACK|GUARD_BINDING_STACK|GUARD_ALIEN_STACK);
     funcall0(function);
@@ -663,8 +688,7 @@ static void attach_os_thread(init_thread_data *scribble)
     block_deferrable_signals(&scribble->oldset);
 #endif
     void* recycled_memory = get_recyclebin_item();
-    struct thread *th = alloc_thread_struct(recycled_memory,
-                                            NO_TLS_VALUE_MARKER_WIDETAG);
+    struct thread *th = alloc_thread_struct(recycled_memory, NO_TLS_VALUE_MARKER);
 
 #ifndef LISP_FEATURE_SB_SAFEPOINT
     /* new-lisp-thread-trampoline doesn't like when the GC signal is blocked */
@@ -896,9 +920,10 @@ alloc_thread_struct(void* spaces, lispobj start_routine) {
     memset(th, 0, sizeof *th);
     lispobj* ptr = (lispobj*)(th + 1);
     lispobj* end = (lispobj*)((char*)th + dynamic_values_bytes);
-    while (ptr < end) *ptr++ = NO_TLS_VALUE_MARKER_WIDETAG;
+    memset(ptr, NO_TLS_VALUE_MARKER & 0xFF, (char*)end-(char*)ptr);
     th->tls_size = dynamic_values_bytes;
 #endif
+
     __attribute((unused)) lispobj* tls = (lispobj*)th;
 #ifdef THREAD_T_NIL_CONSTANTS_SLOT
     tls[THREAD_T_NIL_CONSTANTS_SLOT] = (NIL << 32) | T;
@@ -981,11 +1006,17 @@ alloc_thread_struct(void* spaces, lispobj start_routine) {
     th->state_word.sprof_enable = 0;
     th->state_word.user_thread_p = 1;
 
-#ifdef ALIEN_STACK_GROWS_DOWNWARD
-    th->alien_stack_pointer=(lispobj*)((char*)th->alien_stack_start
-                                       + ALIEN_STACK_SIZE-N_WORD_BYTES);
+    lispobj* alien_stack_end = (lispobj*)((char*)th->alien_stack_start + ALIEN_STACK_SIZE);
+#ifdef ALIEN_STACK_GROWS_UPWARD
+    th->alien_stack_pointer = alien_stack_start;
+#elif defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64
+    /* It is not necessary to subtract a word from the starting stack pointer
+     * because consuming C stack space pre-decrements before using the pointer.
+     * So this choice has nothing to do with saving a word of memory, and everything
+     * to do with my lack of understanding of the conventions for non-x86. */
+    th->alien_stack_pointer = alien_stack_end;
 #else
-    th->alien_stack_pointer=(lispobj*)((char*)th->alien_stack_start);
+    th->alien_stack_pointer = alien_stack_end - 1;
 #endif
 
 #ifdef HAVE_THREAD_PSEUDO_ATOMIC_BITS_SLOT
@@ -1109,7 +1140,7 @@ uword_t create_thread(struct thread_instance* instance, lispobj start_routine)
                      CREATE_SUSPENDED, &tid);
     boolean success = th->os_thread != 0;
     if (success) {
-        instance->primitive_thread = (lispobj)th;
+        instance->uw_primitive_thread = (lispobj)th;
         th->os_kernel_tid = tid;
         ResumeThread((HANDLE)th->os_thread);
     } else {
@@ -1131,7 +1162,7 @@ int release_gc_lock() { return mutex_release(&in_gc_lock); }
 /*
  * (With SB-SAFEPOINT, see the definitions in safepoint.c instead.)
  */
-#ifndef LISP_FEATURE_SB_SAFEPOINT
+#if !defined LISP_FEATURE_SB_SAFEPOINT && !defined STANDALONE_LDB
 
 /* To avoid deadlocks when gc stops the world all clients of each
  * mutex must enable or disable SIG_STOP_FOR_GC for the duration of
@@ -1196,7 +1227,7 @@ void gc_stop_the_world()
             gc_assert(state != STATE_RUNNING);
         }
     }
-    FSHOW_SIGNAL((stderr,"/gc_stop_the_world:end\n"));
+    event0("/gc_stop_the_world:end");
 #ifdef COLLECT_GC_STATS
     clock_gettime(CLOCK_MONOTONIC, &stw_end_time);
     stw_elapsed = (stw_end_time.tv_sec - stw_begin_time.tv_sec)*1000000000L
@@ -1298,7 +1329,7 @@ void wake_thread(struct thread_instance* lispthread)
          * that leave_region safepoint will acknowledge the signal, so
          * there is no need to take locks, roll thread to safepoint
          * etc. */
-        struct thread* thread = (void*)lispthread->primitive_thread;
+        struct thread* thread = (void*)lispthread->uw_primitive_thread;
         if (thread == get_sb_vm_thread()) {
             sb_pthr_kill(thread, 1); // can't fail
             check_pending_thruptions(NULL);
@@ -1318,7 +1349,7 @@ void wake_thread(struct thread_instance* lispthread)
 #elif defined LISP_FEATURE_SB_SAFEPOINT
     wake_thread_impl(lispthread);
 #else
-    pthread_kill(lispthread->os_thread, SIGURG);
+    pthread_kill(lispthread->uw_os_thread, SIGURG);
 #endif
 }
 #endif
