@@ -66,6 +66,7 @@
 #include <wincrypt.h>
 #include <stdarg.h>
 #include <string.h>
+#include "print.h"
 
 /* missing definitions for modern mingws */
 #ifndef EH_UNWINDING
@@ -86,126 +87,6 @@ typedef WCHAR console_char;
 typedef CHAR console_char;
 #endif
 
-/* wrappers for winapi calls that must be successful (like SBCL's
- * (aver ...) form). */
-
-/* win_aver function: basic building block for miscellaneous
- * ..AVER.. macrology (below) */
-
-/* To do: These routines used to be "customizable" with dyndebug_init()
- * and variables like dyndebug_survive_aver, dyndebug_skip_averlax based
- * on environment variables.  Those features got lost on the way, but
- * ought to be reintroduced. */
-
-static inline
-intptr_t win_aver(intptr_t value, char* comment, char* file, int line,
-                  int justwarn)
-{
-    if (!value) {
-        LPSTR errorMessage = "<FormatMessage failed>";
-        DWORD errorCode = GetLastError(), allocated=0;
-        int posixerrno = errno;
-        const char* posixstrerror = strerror(errno);
-        char* report_template =
-            "Expression unexpectedly false: %s:%d\n"
-            " ... %s\n"
-            "     ===> returned #X%p, \n"
-            "     (in thread %p)"
-            " ... Win32 thinks:\n"
-            "     ===> code %u, message => %s\n"
-            " ... CRT thinks:\n"
-            "     ===> code %u, message => %s\n";
-
-        allocated =
-            FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER|
-                           FORMAT_MESSAGE_FROM_SYSTEM,
-                           NULL,
-                           errorCode,
-                           MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US),
-                           (LPSTR)&errorMessage,
-                           1024u,
-                           NULL);
-
-        if (justwarn) {
-            fprintf(stderr, report_template,
-                    file, line,
-                    comment, value,
-                    get_sb_vm_thread(),
-                    (unsigned)errorCode, errorMessage,
-                    posixerrno, posixstrerror);
-        } else {
-            lose(report_template,
-                    file, line,
-                    comment, value,
-                    get_sb_vm_thread(),
-                    (unsigned)errorCode, errorMessage,
-                    posixerrno, posixstrerror);
-        }
-        if (allocated)
-            LocalFree(errorMessage);
-    }
-    return value;
-}
-
-/* sys_aver function: really tiny adaptor of win_aver for
- * "POSIX-parody" CRT results ("lowio" and similar stuff):
- * negative number means something... negative. */
-static inline
-intptr_t sys_aver(long value, char* comment, char* file, int line,
-              int justwarn)
-{
-    win_aver((intptr_t)(value>=0),comment,file,line,justwarn);
-    return value;
-}
-
-/* Check for (call) result being boolean true. (call) may be arbitrary
- * expression now; massive attack of gccisms ensures transparent type
- * conversion back and forth, so the type of AVER(expression) is the
- * type of expression. Value is the same _if_ it can be losslessly
- * converted to (void*) and back.
- *
- * Failed AVER() is normally fatal. Well, unless dyndebug_survive_aver
- * flag is set. */
-
-#define AVER(call)                                                      \
-    ({ __typeof__(call) __attribute__((unused)) me =                    \
-            (__typeof__(call))                                          \
-            win_aver((intptr_t)(call), #call, __FILE__, __LINE__, 0);      \
-        me;})
-
-/* AVERLAX(call): do the same check as AVER did, but be mild on
- * failure: print an annoying unrequested message to stderr, and
- * continue. With dyndebug_skip_averlax flag, AVERLAX stop even to
- * check and complain. */
-
-#define AVERLAX(call)                                                   \
-    ({ __typeof__(call) __attribute__((unused)) me =                    \
-            (__typeof__(call))                                          \
-            win_aver((intptr_t)(call), #call, __FILE__, __LINE__, 1);      \
-        me;})
-
-/* Now, when failed AVER... prints both errno and GetLastError(), two
- * variants of "POSIX/lowio" style checks below are almost useless
- * (they build on sys_aver like the two above do on win_aver). */
-
-#define CRT_AVER_NONNEGATIVE(call)                              \
-    ({ __typeof__(call) __attribute__((unused)) me =            \
-            (__typeof__(call))                                  \
-            sys_aver((call), #call, __FILE__, __LINE__, 0);     \
-        me;})
-
-#define CRT_AVERLAX_NONNEGATIVE(call)                           \
-    ({ __typeof__(call) __attribute__((unused)) me =            \
-            (__typeof__(call))                                  \
-            sys_aver((call), #call, __FILE__, __LINE__, 1);     \
-        me;})
-
-/* to be removed */
-#define CRT_AVER(booly)                                         \
-    ({ __typeof__(booly) __attribute__((unused)) me = (booly);  \
-        sys_aver((booly)?0:-1, #booly, __FILE__, __LINE__, 0);  \
-        me;})
-
 /* The exception handling function looks like this: */
 EXCEPTION_DISPOSITION handle_exception(EXCEPTION_RECORD *,
                                        struct lisp_exception_frame *,
@@ -238,8 +119,8 @@ static void set_seh_frame(void *frame)
 
 void alloc_gc_page()
 {
-    AVER(VirtualAlloc(GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
-                      MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE));
+    gc_assert(VirtualAlloc(GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
+                           MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE));
 }
 
 /* Permit loads from GC_SAFEPOINT_PAGE_ADDR (NB page state change is
@@ -266,15 +147,15 @@ void alloc_gc_page()
 void map_gc_page()
 {
     DWORD oldProt;
-    AVER(VirtualProtect((void*) GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
-                        PAGE_READWRITE, &oldProt));
+    gc_assert(VirtualProtect((void*) GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
+                             PAGE_READWRITE, &oldProt));
 }
 
 void unmap_gc_page()
 {
     DWORD oldProt;
-    AVER(VirtualProtect((void*) GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
-                        PAGE_NOACCESS, &oldProt));
+    gc_assert(VirtualProtect((void*) GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
+                             PAGE_NOACCESS, &oldProt));
 }
 
 #endif
@@ -786,8 +667,8 @@ set_up_win64_seh_thunk(size_t page_size)
     if (page_size < sizeof(struct win64_seh_data))
         lose("Not enough space to allocate struct win64_seh_data");
 
-    AVER(VirtualAlloc(WIN64_SEH_DATA_ADDR, page_size,
-                      MEM_RESERVE | MEM_COMMIT, PAGE_EXECUTE_READWRITE));
+    gc_assert(VirtualAlloc(WIN64_SEH_DATA_ADDR, page_size,
+                           MEM_RESERVE | MEM_COMMIT, PAGE_EXECUTE_READWRITE));
 
     struct win64_seh_data *seh_data = (void *) WIN64_SEH_DATA_ADDR;
     DWORD64 base = (DWORD64) seh_data;
@@ -816,7 +697,7 @@ set_up_win64_seh_thunk(size_t page_size)
     volatile uint8_t *tramp = seh_data->handler_trampoline;
     tramp[0] = 0xFF; // jmp qword ptr [rip+2]
     tramp[1] = 0x25;
-    UNALIGNED_STORE32((tramp+2), 2);
+    UNALIGNED_STORE32((void*volatile)(tramp+2), 2);
     tramp[6] = 0x66; // 2-byte nop
     tramp[7] = 0x90;
     *(void **)(tramp+8) = handle_exception;
@@ -836,7 +717,7 @@ set_up_win64_seh_thunk(size_t page_size)
     rt->EndAddress = 16;
     rt->UnwindData = (DWORD64) ui - base;
 
-    AVER(RtlAddFunctionTable(rt, 1, base));
+    gc_assert(RtlAddFunctionTable(rt, 1, base));
 }
 #endif
 
@@ -888,8 +769,9 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len,
 {
     if (!addr) {
         int protection = attributes & IS_GUARD_PAGE ? PAGE_NOACCESS : PAGE_EXECUTE_READWRITE;
-        return
-            AVERLAX(VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, protection));
+        os_vm_address_t actual = VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, protection);
+        gc_assert(actual);
+        return actual;
     }
 
     os_vm_address_t actual = VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE);
@@ -903,7 +785,8 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len,
             return 0;
         }
 
-        return AVERLAX(VirtualAlloc(NULL, len, MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE));
+        actual = VirtualAlloc(NULL, len, MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE);
+        gc_assert(actual);
     }
 
     return actual;
@@ -923,27 +806,27 @@ os_validate_nocommit(int attributes, os_vm_address_t addr, os_vm_size_t len)
             fflush(stderr);
             return 0;
         }
-        return AVERLAX(VirtualAlloc(NULL, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE));
+        actual = VirtualAlloc(NULL, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
+        gc_assert(actual);
     }
 
     return actual;
 }
 
-void* os_commit_memory(os_vm_address_t addr, os_vm_size_t len)
+void os_commit_memory(os_vm_address_t addr, os_vm_size_t len)
 {
-    return
-        AVERLAX(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE));
+    if (len) gc_assert(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE));
 }
 
-
-void os_revalidate_bzero(os_vm_address_t addr,  os_vm_size_t len) {
-    AVERLAX(VirtualFree(addr, len, MEM_DECOMMIT));
+void os_decommit_mem(os_vm_address_t addr,  os_vm_size_t len) {
+    gc_assert(gc_active_p);
+    gc_assert(VirtualFree(addr, len, MEM_DECOMMIT));
 }
 
 void
 os_invalidate(os_vm_address_t addr, os_vm_size_t len)
 {
-    AVERLAX(VirtualFree(addr, 0, MEM_RELEASE));
+    gc_assert(VirtualFree(addr, 0, MEM_RELEASE));
 }
 
 /*
@@ -955,26 +838,33 @@ os_invalidate(os_vm_address_t addr, os_vm_size_t len)
  * a lazy read (demand page) setup, but that would mean keeping an
  * open file pointer for the core indefinately (and be one more
  * thing to maintain).
+ * FIXME: I would bet that we can use PAGE_EXECUTE_WRITECOPY for this,
+ * but I'll leave it to someone who actually cares.
  */
 
 void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len,
-                      int __attribute__((unused)) execute)
+                      int is_readonly_space)
 {
     os_commit_memory(addr, len);
 #ifdef LISP_FEATURE_64_BIT
-    CRT_AVER_NONNEGATIVE(_lseeki64(fd, offset, SEEK_SET));
+    os_vm_offset_t res = _lseeki64(fd, offset, SEEK_SET);
 #else
-    CRT_AVER_NONNEGATIVE(lseek(fd, offset, SEEK_SET));
+    os_vm_offset_t res = lseek(fd, offset, SEEK_SET);
 #endif
+    gc_assert(res == offset);
     size_t count;
 
+    os_vm_address_t original_addr = addr;
+    os_vm_size_t original_len = len;
     while (len) {
         unsigned to_read = len > INT_MAX ? INT_MAX : len;
         count = read(fd, addr, to_read);
         addr += count;
         len -= count;
-        CRT_AVER(count == to_read);
+        gc_assert(count == to_read);
     }
+    DWORD old;
+    if (is_readonly_space) VirtualProtect(original_addr, original_len, PAGE_READONLY, &old);
     return (void*)0;
 }
 static DWORD os_protect_modes[8] = {
@@ -994,9 +884,9 @@ os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
     DWORD old_prot;
 
     DWORD new_prot = os_protect_modes[prot];
-    AVER(VirtualProtect(address, length, new_prot, &old_prot)||
-         (VirtualAlloc(address, length, MEM_COMMIT, new_prot) &&
-          VirtualProtect(address, length, new_prot, &old_prot)));
+    gc_assert(VirtualProtect(address, length, new_prot, &old_prot)||
+              (VirtualAlloc(address, length, MEM_COMMIT, new_prot) &&
+               VirtualProtect(address, length, new_prot, &old_prot)));
     odxprint(misc,"Protecting %p + %p vmaccess %d "
              "newprot %08x oldprot %08x",
              address,length,prot,new_prot,old_prot);
@@ -1117,7 +1007,6 @@ handle_access_violation(os_context_t *ctx,
 #endif
 
     /* Safepoint pages */
-#ifdef LISP_FEATURE_SB_THREAD
     if (fault_address == (void *) GC_SAFEPOINT_TRAP_ADDR) {
         thread_in_lisp_raised(ctx);
         return 0;
@@ -1127,15 +1016,13 @@ handle_access_violation(os_context_t *ctx,
         thread_in_safety_transition(ctx);
         return 0;
     }
-#endif
 
     /* dynamic space */
     page_index_t page = find_page_index(fault_address);
-    if (page != -1
-#ifndef LISP_FEATURE_SOFT_CARD_MARKS
-        && !PAGE_WRITEPROTECTED_P(page)
-#endif
-        ) {
+#ifdef LISP_FEATURE_SOFT_CARD_MARKS
+    if (page >= 0) lose("should not get access violation in dynamic space");
+#else
+    if (page != -1 && !PAGE_WRITEPROTECTED_P(page)) {
         os_commit_memory(PTR_ALIGN_DOWN(fault_address, os_vm_page_size),
                          os_vm_page_size);
         return 0;
@@ -1143,6 +1030,7 @@ handle_access_violation(os_context_t *ctx,
     if (gencgc_handle_wp_violation(ctx, fault_address)) {
         return 0;
     }
+#endif
 
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     extern int immobile_space_handle_wp_violation(void*);
@@ -1211,7 +1099,7 @@ signal_internal_error_or_lose(os_context_t *ctx,
             (void*)(intptr_t)exception_record->ExceptionCode);
     fprintf(stderr, "Faulting IP: %p.\n",
             (void*)(intptr_t)exception_record->ExceptionAddress);
-    if (exception_record->ExceptionCode == EXCEPTION_ACCESS_VIOLATION) {
+    if ((long int)exception_record->ExceptionCode == EXCEPTION_ACCESS_VIOLATION) {
         MEMORY_BASIC_INFORMATION mem_info;
 
         if (VirtualQuery(fault_address, &mem_info, sizeof mem_info)) {
@@ -1382,12 +1270,11 @@ veh(EXCEPTION_POINTERS *ep)
     }
 
     DWORD64 rip = ep->ContextRecord->Rip;
-    DWORD code = ep->ExceptionRecord->ExceptionCode;
+    long int code = ep->ExceptionRecord->ExceptionCode;
     BOOL from_lisp =
-        (rip >= READ_ONLY_SPACE_START &&
-         rip <= READ_ONLY_SPACE_END) ||
-        (rip >= DYNAMIC_SPACE_START &&
-         rip <= DYNAMIC_SPACE_START+dynamic_space_size);
+        (rip >= DYNAMIC_SPACE_START && rip < DYNAMIC_SPACE_START+dynamic_space_size) ||
+        (rip >= READ_ONLY_SPACE_START && rip < READ_ONLY_SPACE_END) ||
+        (rip >= STATIC_SPACE_START && rip < (uword_t)static_space_free_pointer);
 
     if (code == EXCEPTION_ACCESS_VIOLATION ||
         code == STATUS_HEAP_CORRUPTION ||
@@ -1435,30 +1322,6 @@ wos_install_interrupt_handlers
 #endif
 }
 
-/*
- * The stubs below are replacements for the windows versions,
- * which can -fail- when used in our memory spaces because they
- * validate the memory spaces they are passed in a way that
- * denies our exception handler a chance to run.
- */
-
-void *memmove(void *dest, const void *src, size_t n)
-{
-    if (dest < src) {
-        size_t i;
-        for (i = 0; i < n; i++) *(((char *)dest)+i) = *(((char *)src)+i);
-    } else {
-        while (n--) *(((char *)dest)+n) = *(((char *)src)+n);
-    }
-    return dest;
-}
-
-void *memcpy(void *dest, const void *src, size_t n)
-{
-    while (n--) *(((char *)dest)+n) = *(((char *)src)+n);
-    return dest;
-}
-
 char *dirname(char *path)
 {
     static char buf[PATH_MAX + 1];
@@ -1492,8 +1355,8 @@ socket_input_available(HANDLE socket, long time, long utime)
 
     FD_ZERO(&readfds);
     FD_ZERO(&errfds);
-    FD_SET(socket, &readfds);
-    FD_SET(socket, &errfds);
+    FD_SET((uword_t)socket, &readfds);
+    FD_SET((uword_t)socket, &errfds);
 
     count = select(0, &readfds, NULL, &errfds, &timeout);
     SetLastError(wsaErrno);
