@@ -775,9 +775,12 @@
 
 (defun test-hash-table ()
   (setf (gethash 5 *table*) 13)
-  (gethash 5 *table*))
+  (setf (gethash 6 *table*) 14)
+  (values (gethash 5 *table*)
+          (gethash 6 *table*)))
 
 (with-test (:name (:no-consing :hash-tables))
+  (test-hash-table) ;; initialize all the vectors first
   (assert-no-consing (test-hash-table)))
 
 ;;; Both with-pinned-objects and without-gcing should not cons
@@ -892,11 +895,11 @@
          (b (list x y z))
          (c (list a b)))
     (declare (dynamic-extent c))
-    (values (first c) (second c))))
+    (values a b)))
 (with-test (:name :let-converted-vars-dx-allocated-bug)
-  (multiple-value-bind (i j) (let-converted-vars-dx-allocated-bug 1 2 3)
-    (assert (and (equal i j)
-                 (equal i (list 1 2 3))))))
+  (multiple-value-bind (i j) (let-converted-vars-dx-allocated-bug-default 1 2 3)
+    (assert (equal i j))
+    (assert (equal i (list 1 2 3)))))
 
 ;;; workaround for bug 419 -- real issue remains, but check that the
 ;;; bandaid holds.
@@ -1117,7 +1120,7 @@
     (test `(lambda () (declare (dynamic-extent #'bar)))
           :allow-style-warnings 'style-warning)
     (test `(lambda () (declare (dynamic-extent bar)))
-          :allow-style-warnings 'style-warning)
+          :allow-warnings 'warning)
     (test `(lambda (bar) (cons bar (lambda () (declare (dynamic-extent bar)))))
           :allow-notes 'sb-ext:compiler-note)
     (test `(lambda ()
@@ -1683,3 +1686,26 @@
                 (lambda (j &rest points)
                   (apply continuation  j (mapcar identity points)))))))
     (() '(1 #()) :test #'equalp)))
+
+(with-test (:name :special-bind-removal)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (multiple-value-prog1 (eval a)
+           (let ((* (let ((x (list 1)))
+                      (setf (car x) 1)
+                      x)))
+             (declare (dynamic-extent *)))))
+    ((1) 1)))
+
+(with-test (:name :let-setf-aref)
+  (checked-compile
+   `(lambda ()
+      (declare (optimize debug))
+      (let ((x (let ((z (make-array 3)))
+                 (setf (aref z 0) 10)
+                 (setf (aref z 0) 20)
+                 z)))
+        (declare (dynamic-extent x))
+        (aref x 0)))
+   :allow-notes nil))
