@@ -495,3 +495,10 @@
         (loop for i from 10 to 24
               do
            (assert (= (sb-sys:sap-ref-word sap (ash i sb-vm:word-shift)))))))))
+
+(with-test (:name :rospace-strings
+                  :fails-on :darwin-jit)
+  (let ((err (handler-case (setf (char (opaque-identity (symbol-name '*readtable*)) 0) #\*)
+               (sb-sys:memory-fault-error (c)
+                 (write-to-string c :escape nil)))))
+    (assert (search "modify a read-only object" err))))
