@@ -11,7 +11,7 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-#+sb-devel (sb-ext:exit :code 104) ; packages are not locked for devs
+#+sb-devel (invoke-restart 'run-tests::skip-file) ; packages are not locked for devs
 
 (load "compiler-test-util.lisp")
 
