@@ -34,6 +34,8 @@
            #:scratch-file-name
            #:*scratch-file-prefix*
            #:with-scratch-file
+           #:with-test-directory
+           #:*test-directory*
            #:opaque-identity
            #:runtime #:split-string #:integer-sequence #:shuffle))
 
@@ -79,13 +81,12 @@
   (and (subtypep x y) (subtypep y x)))
 
 (defun ctype= (left right)
-  (let ((a (sb-kernel:specifier-type left)))
+  (let ((a (sb-kernel:values-specifier-type left)))
     ;; SPECIFIER-TYPE is a memoized function, and TYPE= is a trivial
     ;; operation if A and B are EQ.
     ;; To actually exercise the type operation, remove the memoized parse.
     (sb-int:drop-all-hash-caches)
-    (let ((b (sb-kernel:specifier-type right)))
-      (assert (not (eq a b)))
+    (let ((b (sb-kernel:values-specifier-type right)))
       (sb-kernel:type= a b))))
 
 (defmacro assert-tri-eq (expected-result expected-certainp form)
@@ -884,6 +885,27 @@
             (let ((,var ,tempname)) ,@forms) ; rebind, as test might asssign into VAR
          (ignore-errors (delete-file ,tempname))))))
 
+(defvar *test-directory*)
+
+(defun call-with-test-directory (fn)
+  ;; FIXME: this writes into the source directory depending on whether
+  ;; TEST_DIRECTORY has been made to point elsewhere or not.
+  (let ((test-directory (parse-native-namestring (posix-getenv "TEST_DIRECTORY")
+                                                 nil *default-pathname-defaults*
+                                                 :as-directory t)))
+    (ensure-directories-exist test-directory)
+    (unwind-protect
+         (let ((*default-pathname-defaults* test-directory)
+               (*test-directory* test-directory))
+           (funcall fn test-directory))
+      (delete-directory test-directory :recursive t))))
+
+(defmacro with-test-directory ((&optional (test-directory-var (gensym)))
+                               &body body)
+  `(call-with-test-directory (lambda (,test-directory-var)
+                               (declare (ignorable ,test-directory-var))
+                               ,@body)))
+
 ;;; Take a list of lists and assemble them as though they are
 ;;; instructions inside the body of a vop. There is no need
 ;;; to use the INST macro in front of each list.
