@@ -9,7 +9,7 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-#-x86-64 (sb-ext:exit :code 104)
+#-x86-64 (invoke-restart 'run-tests::skip-file)
 
 ;;; This trivial function failed to compile due to rev 88d078fe
 (defun foo (&key k)
@@ -63,16 +63,8 @@
                                       '(lambda () (sb-sys:%primitive tryme))))
                 thereis (search "MOV BYTE PTR GS:[RAX]" line))))
 
-(defun disasm (safety expr &optional (remove-epilogue t))
-  ;; This lambda has a name because if it doesn't, then the name
-  ;; is something stupid like (lambda () in ...) which pretty-prints
-  ;; on a random number of lines.
-  (let ((fun (compile nil
-                      `(sb-int:named-lambda test ()
-                         (declare (optimize (debug 0) (safety ,safety)
-                                            (sb-c:verify-arg-count 0)))
-                         ,expr))))
-    (let ((lines (disassembly-lines fun)))
+(defun strip-assem-junk (fun &optional (remove-epilogue t))
+    (let ((lines (disassembly-lines (compile nil fun))))
       ;; For human-readability, kill the whitespace
       (setq lines (mapcar (lambda (x) (string-left-trim " ;" x)) lines))
       (when (string= (car (last lines)) "")
@@ -87,16 +79,35 @@
                       '("MOV RSP, RBP" "CLC" "POP RBP" "RET")
                       (subseq lines (- (length lines) 4))))
           (butlast lines 4)
-          lines))))
+          lines)))
+(defun disasm-load (safety symbol)
+  ;; This lambda has a name because if it doesn't, then the name
+  ;; is something stupid like (lambda () in ...) which pretty-prints
+  ;; on a random number of lines.
+  (strip-assem-junk   `(sb-int:named-lambda test ()
+                         (declare (optimize (debug 0) (safety ,safety)
+                                            (sb-c:verify-arg-count 0)))
+                         ,symbol)))
+(defun disasm-store (sexpr)
+  (strip-assem-junk   `(sb-int:named-lambda test (x)
+                         (declare (ignorable x))
+                         (declare (optimize (debug 0)
+                                            (sb-c:verify-arg-count 0)))
+                         ,sexpr)))
 
 (with-test (:name :symeval-known-thread-local
             :skipped-on (not :sb-thread))
   ;; It should take 1 instruction to read a known thread-local var
-  (assert (= (length (disasm 1 'sb-thread:*current-thread*)) 1))
-  (assert (= (length (disasm 1 'sb-sys:*interrupt-pending*)) 1))
-  (assert (= (length (disasm 1 'sb-kernel:*gc-inhibit*)) 1))
-  (assert (= (length (disasm 1 'sb-kernel:*restart-clusters*)) 1))
-  (assert (= (length (disasm 1 'sb-kernel:*handler-clusters*)) 1)))
+  (assert (= (length (disasm-load 1 'sb-thread:*current-thread*)) 1))
+  (assert (= (length (disasm-load 1 'sb-sys:*interrupt-pending*)) 1))
+  (assert (= (length (disasm-load 1 'sb-kernel:*gc-inhibit*)) 1))
+  (assert (= (length (disasm-load 1 'sb-kernel:*restart-clusters*)) 1))
+  (assert (= (length (disasm-load 1 'sb-kernel:*handler-clusters*)) 1)))
+
+(with-test (:name :set-known-thread-local :skipped-on (or (not :immobile-space)
+                                                          (not :sb-thread)))
+  ;; It should take 1 instruction to write a known thread-local var
+  (assert (= (length (disasm-store '(setq sb-kernel:*gc-inhibit* x))) 1)))
 
 ;; Lack of earmuffs on this symbol allocates it in dynamic space
 (defvar foo)
@@ -113,7 +124,7 @@
   ;;    83FA61             CMP EDX, 97
   ;;    480F44142538F94B20 CMOVEQ RDX, [#x204BF938]  ; *PRINT-BASE*
   ;; (TODO: could use "CMOVEQ RDX, [RIP-n]" in immobile code)
-  (let ((text (disasm 0 '*print-base*)))
+  (let ((text (disasm-load 0 '*print-base*)))
     (assert (= (length text) 3)) ; number of lines
     ;; two lines should be annotated with *PRINT-BASE*
     (assert (= (loop for line in text count (search "*PRINT-BASE*" line)) 2)))
@@ -123,7 +134,7 @@
   ;;    488B059EFFFFFF     MOV RAX, [RIP-98]         ; 'FOO
   ;;    83FA61             CMP EDX, 97
   ;;    480F4450F9         CMOVEQ RDX, [RAX-7]
-  (let ((text (disasm 0 'foo)))
+  (let ((text (disasm-load 0 'foo)))
     (assert (= (length text) 4))
     ;; two lines should be annotated with FOO
     (assert (= (loop for line in text count (search "FOO" line)) 2))))
@@ -142,7 +153,7 @@
   ;;    83FA61             CMP EDX, 97
   ;;    480F44142518A24C20 CMOVEQ RDX, [#x204CA218] ; *BLUB*
   ;; (TODO: could use "CMOVEQ RDX, [RIP-n]" in immobile code)
-  (let ((text (disasm 0 '*blub*)))
+  (let ((text (disasm-load 0 '*blub*)))
     (assert (= (length text) 4))
     ;; two lines should be annotated with *BLUB*
     (assert (= (loop for line in text count (search "*BLUB*" line)) 2)))
@@ -153,7 +164,7 @@
   ;;    4A8B142A           MOV RDX, [RDX+R13]
   ;;    83FA61             CMP EDX, 97
   ;;    480F4450F9         CMOVEQ RDX, [RAX-7]
-  (assert (= (length (disasm 0 'blub)) 5)))
+  (assert (= (length (disasm-load 0 'blub)) 5)))
 
 (with-test (:name :object-not-type-error-encoding)
   ;; There should not be a "MOV Rnn, #xSYMBOL" instruction
@@ -392,7 +403,7 @@
   (assert (thing-ref-thing-ref (make-thing :x (make-thing :x 3)) 'foo)))
 
 (with-test (:name :huge-code :skipped-on (not :immobile-code))
-  (sb-vm::allocate-code-object :immobile 0 4 (* 2 1024 1024)))
+  (sb-vm::allocate-code-object :immobile 4 (* 2 1024 1024)))
 
 (defun bbb (x y z)
   ;; I don't want the number of expected comparisons to depend on whether
@@ -635,16 +646,6 @@
                ;; n-widetags divided by 4, plus jump table count word.
                65)))
 
-sb-vm::(define-vop (cl-user::test)
-  (:generator 0
-   ;; pointless to resize to :qword, but the rule won't try to apply itself
-   ;; to :dword because zeroing the upper 32 bits is a visible effect.
-   (inst mov (reg-in-size rax-tn :qword) (reg-in-size rcx-tn :qword))
-   (inst mov rax-tn rcx-tn)))
-(with-test (:name :mov-mov-elim-ignore-resized-reg
-                  :fails-on :sbcl) ; just don't crash
-  (checked-compile '(lambda () (sb-sys:%primitive test) 0)))
-
 (defstruct a)
 (defstruct (achild (:include a)))
 (defstruct (agrandchild (:include achild)))
@@ -903,7 +904,24 @@ sb-vm::(define-vop (cl-user::test)
   ;; and some range-based testing rather than just 2 EQ tests.
   (check-integerp-cmp-opcodes 2 '(integer 1 2)))
 
-(defun typep-asm-code-length (type)
+(defun show-pretty-lines (type lines)
+  (format t ";;;; Type: ~s~%" type)
+  (dolist (line lines)
+    (when (plusp (length line))
+      (let* ((label
+              (if (char= (char line 0) #\L)
+                  (subseq line 0 (1+ (position #\: line)))
+                  ""))
+             (opcode-bytes
+              (progn
+                (setq line (string-left-trim " " (subseq line (length label))))
+                (subseq line 0 (position #\Space line)))))
+        (setq line (string-left-trim " " (subseq line (length opcode-bytes))))
+        (format t "; ~4a~20a~a~%"
+                label opcode-bytes line))))
+  (terpri))
+
+(defun typep-asm-code-length (type &optional print)
   (let* ((lines
           (disassembly-lines
            (compile nil
@@ -914,6 +932,7 @@ sb-vm::(define-vop (cl-user::test)
           (some (lambda (x) (or (search "#<FUNCTION" x)
                                 (search "#<FDEFN" x)))
                 lines)))
+    (when print (show-pretty-lines type lines))
     (values (length lines) callp)))
 
 ;;; Counting instructions of assembly is sort of a very rough guess
@@ -939,7 +958,48 @@ sb-vm::(define-vop (cl-user::test)
 (write-golden-typep-data "../interesting-types.lisp-expr"
                          "typep-golden-data.txt")
 
-(defun compare-to-golden-typep-data (pathname)
+;;; FIXME: Some time after the MANY-INTERESTING-ARRAY-TYPES test
+;;; got disabled, the code size changed either due to a regression,
+;;; or more aggressive inlining of ">=" and "<="
+;;; which now causes these warnings. Figure out the new baseline.
+;;
+;; WARNING: (AND INTEGER (NOT (SIGNED-BYTE 31))) was 32 is 38
+;; WARNING: (AND INTEGER (NOT (SIGNED-BYTE 32))) was 32 is 38
+;; WARNING: (INTEGER (0) *) was 18 is 22
+;; WARNING: (INTEGER * -1) was 18 is 22
+;; WARNING: (INTEGER * -1000000) was 18 is 22
+;; WARNING: (INTEGER * -1073741825) was 18 is 23
+;; WARNING: (INTEGER * -12) was 18 is 22
+;; WARNING: (INTEGER * -14) was 18 is 22
+;; WARNING: (INTEGER * -17) was 18 is 22
+;; WARNING: (INTEGER * -2) was 18 is 22
+;; WARNING: (INTEGER * -2147483649) was 18 is 23
+;; WARNING: (INTEGER * -4611686018427387901) was 18 is 23
+;; WARNING: (INTEGER * -54043195528445951) was 18 is 23
+;; WARNING: (INTEGER * -541073411) was 18 is 22
+;; WARNING: (INTEGER * -6) was 18 is 22
+;; WARNING: (INTEGER * -9) was 18 is 22
+;; WARNING: (INTEGER * 0) was 24 is 22
+;; WARNING: (INTEGER * 20) was 18 is 22
+;; WARNING: (INTEGER * 2047) was 18 is 22
+;; WARNING: (INTEGER * 2147483647) was 18 is 23
+;; WARNING: (INTEGER * 4611686018427387900) was 18 is 23
+;; WARNING: (INTEGER * 576460752303423487) was 18 is 23
+;; WARNING: (INTEGER * 65535) was 18 is 22
+;; WARNING: (INTEGER -1) was 18 is 22
+;; WARNING: (INTEGER 1) was 18 is 22
+;; WARNING: (INTEGER 1073741824) was 18 is 23
+;; WARNING: (INTEGER 1152921504606846961) was 18 is 23
+;; WARNING: (INTEGER 1899) was 18 is 22
+;; WARNING: (INTEGER 2) was 18 is 22
+;; WARNING: (INTEGER 2147483648) was 18 is 23
+;; WARNING: (INTEGER 2305843009214) was 18 is 23
+;; WARNING: (INTEGER 4611686018427387900) was 18 is 23
+;; WARNING: (INTEGER 4611686018427387901) was 18 is 23
+;; WARNING: (INTEGER 4611686018427387902) was 18 is 23
+;; WARNING: (INTEGER 4611686018427387903) was 18 is 23
+;; WARNING: (INTEGER 63) was 18 is 22
+(defun compare-to-golden-typep-data (pathname &optional print)
   (with-open-file (input pathname)
     (let ((*package* (find-package "SB-KERNEL")))
       (loop (let ((line (read-line input nil input)))
@@ -947,13 +1007,15 @@ sb-vm::(define-vop (cl-user::test)
               (with-input-from-string (stream line :start 2)
                 (let ((expect-n (read stream))
                       (type (read stream)))
-                  (multiple-value-bind (linecount callp) (typep-asm-code-length type)
+                  (multiple-value-bind (linecount callp)
+                      (typep-asm-code-length type print)
                     (declare (ignore callp))
                     (when (/= linecount expect-n)
                       (warn "~S was ~d is ~d" type expect-n linecount))))))))))
 
-(with-test (:name :many-interesting-array-types :skipped-on (or (:not :sb-unicode)
-                                                                (not :sb-devel)))
+(with-test (:name :many-interesting-array-types
+                  :skipped-on (:or (:not :sb-unicode)
+                                   (:not :immobile-space)))
   (compare-to-golden-typep-data "typep-golden-data.txt"))
 
 (with-test (:name :integerp->bignump-strength-reduction)
@@ -1142,3 +1204,86 @@ sb-vm::(define-vop (cl-user::test)
 (defun zook ()
   (sb-sys:%primitive trythis)
   nil)
+
+;;; Previously INITIALIZE-VECTOR would move every element into
+;;; a register, causing each to require a store barrier
+;;; and then a register-to-memory move like so:
+;;;     BB02000000       MOV EBX, 2
+;;;     488D4601         LEA RAX, [RSI+1]
+;;;     48C1E80A         SHR RAX, 10
+;;;     25FFFF0F00       AND EAX, 1048575
+;;;     41C6040400       MOV BYTE PTR [R12+RAX], 0
+;;;     48895E01         MOV [RSI+1], RBX
+;;; The improved code emits 4 consecutive MOV instructions,
+;;; one per item. It is still suboptimal in that it can not discern
+;;; between initializing and updating, so it always uses a :QWORD move
+;;; despite the prezeroed pages.
+(with-test (:name :init-vector-mov-to-mem)
+  (let* ((lines (disassembly-lines
+                 '(lambda () (vector #\x 1 2 3))))
+         (magic-value
+          (write-to-string
+           (logior (ash (char-code #\x) 8) sb-vm:character-widetag)))
+         (start
+          (position-if
+           (lambda (line) (search magic-value line))
+           lines)))
+    (assert start)
+    (assert (search ", 2" (nth (+ start 1) lines)))
+    (assert (search ", 4" (nth (+ start 2) lines)))
+    (assert (search ", 6" (nth (+ start 3) lines)))))
+
+(defglobal *myglobalvar* 3)
+(with-test (:name :disassemble-symbol-global-value)
+  (assert (loop for line in (disassembly-lines '(lambda () *myglobalvar*))
+                thereis (search "*MYGLOBALVAR*" line))))
+
+(defun compiler-trace-output-lines (lexpr)
+  (let ((string-stream (make-string-output-stream)))
+    (let ((sb-c::*compiler-trace-output* string-stream)
+          (sb-c::*compile-trace-targets* '(:vop))
+          (*print-pretty* nil))
+      (compile nil lexpr))
+    (split-string (get-output-stream-string string-stream)
+                  #\newline)))
+
+(defun assert-has-gc-barrier (match-string lines &optional (skip 0))
+
+  (let (found)
+;; Look for something like this in the trace output:
+;; "VOP CLOSURE-INIT t9[RBX(d)] :NORMAL t13[RSI(d)] :NORMAL {0} "
+;; "        MOV     0, #<TN t14[RAX(u)] :NORMAL>, #<TN t9[RBX(d)] :NORMAL>"
+;; "        SHR     0, #<TN t14[RAX(u)] :NORMAL>, 10"
+;; "        AND     6, #<TN t14[RAX(u)] :NORMAL>, #S(FIXUP :NAME NIL :FLAVOR GC-BARRIER :OFFSET 0)"
+;; "        MOV     4, PTR [R12+RAX+0], 0"
+;; "        MOV     7, PTR [RBX+5], #<TN t13[RSI(d)] :NORMAL>"
+    (loop while lines
+          do
+          (when (and (search match-string (car lines))
+                     (minusp (decf skip)))
+            (setq found t)
+            (let ((barrier-fixup-line (nth 3 lines)))
+              (assert (search ":FLAVOR GC-BARRIER" barrier-fixup-line))
+              (return)))
+          (pop lines))
+    ;; Fail if we didn't find the sentinel CLOSURE-INIT
+    (assert found)))
+
+(with-test (:name :closure-init-gc-barrier)
+  (let ((lines
+         (compiler-trace-output-lines
+           '(lambda (address)
+             (declare (sb-vm:word address))
+             (let ((sap (sb-sys:int-sap address)))
+               (lambda () sap))))))
+    (assert-has-gc-barrier "CLOSURE-INIT" lines)))
+
+(defstruct (point (:constructor make-point (x))) x (y 0) (z 0))
+(with-test (:name :structure-init-gc-barrier)
+  (let ((lines
+          (compiler-trace-output-lines
+           '(lambda (val)
+             (declare (double-float val) (inline make-point))
+             (let ((neg (- val))) (make-point neg))))))
+    (assert-has-gc-barrier "SET-SLOT" lines
+                           #-compact-instance-header 1)))
