@@ -78,7 +78,7 @@
   (find id (cdr spaces) :key #'space-id))
 (defun compute-nil-object (spaces)
   (let ((space (get-space static-core-space-id spaces)))
-    (%make-lisp-obj (logior (space-addr space) #x117))))
+    (%make-lisp-obj (logior (space-addr space) #x117)))) ; SUPER KLUDGE
 
 ;;; Given OBJ which is tagged pointer into the target core, translate it into
 ;;; the range at which the core is now mapped during execution of this tool,
@@ -103,6 +103,8 @@
                  (:constructor %make-core))
   (spaces)
   (nil-object)
+  ;; mapping from small integer ID to package
+  (pkg-id->package)
   ;; mapping from string naming a package to list of symbol names (strings)
   ;; that are external in the package.
   (packages (make-hash-table :test 'equal))
@@ -130,7 +132,7 @@
 
 (defglobal *editcore-ppd*
   ;; copy no entries for macros/special-operators (flet, etc)
-  (let ((ppd (sb-pretty::make-pprint-dispatch-table nil nil nil)))
+  (let ((ppd (sb-pretty::make-pprint-dispatch-table #() nil nil)))
     (set-pprint-dispatch 'string
                          ;; Write strings without string quotes
                          (lambda (stream string) (write-string string stream))
@@ -244,11 +246,12 @@
          (let ((p (position #\/ x :from-end t)))
            (if p (subseq x (1+ p)) x)))
         ((symbolp x)
-         (let ((name (translate (symbol-name x) spaces)))
-           (when (eq (symbol-package x) core-nil) ; uninterned
+         (let ((package-id (sb-impl::symbol-package-id x))
+               (name (translate (symbol-name x) spaces)))
+           (when (eq package-id 0) ; uninterned
              (return-from recurse (string-downcase name)))
            (let* ((package (truly-the package
-                                      (translate (symbol-package x) spaces)))
+                                      (aref (core-pkg-id->package core) package-id)))
                   (package-name (translate (package-%name package) spaces)))
              ;; The name-cleaning code wants to compare against symbols
              ;; in CL, PCL, and KEYWORD, so use real symbols for those.
@@ -330,6 +333,8 @@
   ;; but no more so than everything else in this file)
   (let ((locs (sb-c::compiled-debug-fun-encoded-locs
                (truly-the sb-c::compiled-debug-fun compiled-debug-fun))))
+    (when (consp locs)
+      (setq locs (cdr (translate locs spaces))))
     (sb-c::compiled-debug-fun-offset
      (sb-c::make-compiled-debug-fun
       :name nil
@@ -393,8 +398,10 @@
           (pairs (cons (descriptorize (svref cells i))
                        (descriptorize (svref cells (1+ i))))))))))
 
+(defmacro package-id (name) (sb-impl::package-id (find-package name)))
+
 ;;; Return either the physical or logical address of the specified symbol.
-(defun find-target-symbol (package-name symbol-name spaces
+(defun find-target-symbol (package-id symbol-name spaces
                            &optional (address-mode :physical))
   (dolist (id `(,immobile-fixedobj-core-space-id ,static-core-space-id))
     (let* ((space (get-space id spaces))
@@ -404,15 +411,10 @@
      (loop
       (when (>= physaddr end) (return))
       (let* ((obj (reconstitute-object (ash physaddr (- n-fixnum-tag-bits))))
-             (size (sb-vm::primitive-object-size obj)))
+             (size (primitive-object-size obj)))
         (when (and (symbolp obj)
                    (string= symbol-name (translate (symbol-name obj) spaces))
-                   (%instancep (symbol-package obj))
-                   (string= package-name
-                            (translate
-                             (package-%name
-                              (truly-the package (translate (symbol-package obj) spaces)))
-                             spaces)))
+                   (= (sb-impl::symbol-package-id obj) package-id))
           (return-from find-target-symbol
             (%make-lisp-obj
                    (logior (ecase address-mode
@@ -420,14 +422,14 @@
                              (:logical (+ (space-addr space) (- physaddr start))))
                            other-pointer-lowtag))))
         (incf physaddr size)))))
-  (bug "Can't find symbol ~A::~A" package-name symbol-name))
+  (bug "Can't find symbol ~A::~A" package-id symbol-name))
 
 (defparameter label-prefix (if (member :darwin *features*) "_" ""))
 (defun labelize (x) (concatenate 'string label-prefix x))
 
 (defun compute-linkage-symbols (spaces)
   (let* ((linkage-info (symbol-global-value
-                        (find-target-symbol "SB-SYS" "*LINKAGE-INFO*"
+                        (find-target-symbol (package-id "SB-SYS") "*LINKAGE-INFO*"
                                             spaces :physical)))
          (hashtable (car (translate linkage-info spaces)))
          (pairs (target-hash-table-alist hashtable spaces))
@@ -452,12 +454,12 @@
   (let* ((linkage-bounds
           (make-bounds
            (symbol-global-value
-            (find-target-symbol "SB-VM" "LINKAGE-TABLE-SPACE-START" spaces :physical))
+            (find-target-symbol (package-id "SB-VM") "LINKAGE-TABLE-SPACE-START" spaces :physical))
            (symbol-global-value
-            (find-target-symbol "SB-VM" "LINKAGE-TABLE-SPACE-END" spaces :physical))))
+            (find-target-symbol (package-id "SB-VM") "LINKAGE-TABLE-SPACE-END" spaces :physical))))
          (linkage-entry-size
           (symbol-global-value
-           (find-target-symbol "SB-VM" "LINKAGE-TABLE-ENTRY-SIZE"
+           (find-target-symbol (package-id "SB-VM") "LINKAGE-TABLE-ENTRY-SIZE"
                                spaces :physical)))
          (linkage-symbols (compute-linkage-symbols spaces))
          (nil-object (compute-nil-object spaces))
@@ -472,11 +474,13 @@
            :linkage-bounds linkage-bounds
            :linkage-entry-size linkage-entry-size
            :linkage-symbols linkage-symbols
-           :linkage-symbol-usedp (make-array (length linkage-symbols) :element-type 'bit)
+           :linkage-symbol-usedp (make-array (length linkage-symbols) :element-type 'bit
+                                             :initial-element 0)
            :enable-pie enable-pie)))
     (let ((package-table
            (symbol-global-value
-            (find-target-symbol "SB-KERNEL" "*PACKAGE-NAMES*" spaces :physical)))
+            (find-target-symbol (package-id "SB-KERNEL") "*PACKAGE-NAMES*" spaces :physical)))
+          (package-alist)
           (symbols (make-hash-table :test 'equal)))
       (dovector (x (translate (%instance-ref (translate package-table spaces) 0) spaces))
         (when (%instancep x) ; package
@@ -486,8 +490,14 @@
                       (pushnew (get-lisp-obj-address sym) (gethash str symbols)))
                     table core)))
             (let ((package (truly-the package (translate x spaces))))
+              (push (cons (sb-impl::package-id package) package) package-alist)
               (scan (package-external-symbols package))
               (scan (package-internal-symbols package))))))
+      (let ((package-by-id (make-array (1+ (reduce #'max package-alist :key #'car))
+                                       :initial-element nil)))
+        (loop for (id . package) in package-alist
+              do (setf (aref package-by-id id) package))
+        (setf (core-pkg-id->package core) package-by-id))
       (dohash ((string symbols) symbols)
         (when (cdr symbols)
           (setf (gethash string ambiguous-symbols) t))))
@@ -576,7 +586,7 @@
                                       (- operand (bounds-low (core-code-bounds core))))))
                    (push (list* (dstate-cur-offs dstate) 5 "mov" text) list)))
                 ((and (eq (inst-name inst) 'mov) ; match "mov qword ptr [R+disp8], imm32"
-                      (eql (sap-ref-8 sap (1- offs))    #x48)
+                      (member (sap-ref-8 sap (1- offs)) '(#x48 #x49)) ; REX.w and maybe REX.b
                       (eql (sap-ref-8 sap offs)         #xC7)
                       ;; modRegRm = #b01 #b000 #b___
                       (eql (logand (sap-ref-8 sap (1+ offs)) #o370) #o100))
@@ -887,7 +897,7 @@
                   collect (sap-ref-8 text-sap i)))
     (when additional-relative-fixups
       (binding* ((existing-fixups (sb-vm::%code-fixups code))
-                 ((absolute relative)
+                 ((absolute relative immediate)
                   (sb-c::unpack-code-fixup-locs
                    (if (fixnump existing-fixups)
                        existing-fixups
@@ -902,9 +912,8 @@
                                                    sb-vm:word-shift))))
                                 additional-relative-fixups)
                         #'<)))
-        (sb-c::pack-code-fixup-locs
-         absolute
-         (merge 'list relative new-sorted #'<))))))
+        (sb-c:pack-code-fixup-locs
+         absolute (merge 'list relative new-sorted #'<) immediate)))))
 
 (defconstant +gf-name-slot+ 5)
 
@@ -951,9 +960,6 @@
           (pp-state (cons (make-hash-table :test 'equal) nil))
           (prev-namestring "")
           (n-linker-relocs 0)
-          (seen-fdefns nil)
-          (seen-trampolines nil)
-          (seen-gfs nil)
           (temp-output (make-string-output-stream :element-type 'base-char))
           end-loc)
   (labels ((dumpwords (sap count stream &optional (exceptions #()) logical-addr)
@@ -1017,12 +1023,14 @@
                        (car (translate (sb-vm::%%code-debug-info code-component) spaces))
                        spaces))
               #'< :key #'cadr)))
-        ;; Possibly a padding word
+        ;; Possibly unboxed words and/or padding
         (let ((here (ash jump-table-count sb-vm:word-shift))
               (first-entry-point (cadar name->addr)))
           (when (> first-entry-point here)
-            (assert (= first-entry-point (+ here 8)))
-            (format output " .quad 0~%")))
+            (format output " .quad ~{0x~x~^,~}~%"
+                    (loop for offs = here then (+ offs 8)
+                          while (< offs first-entry-point)
+                          collect (sap-ref-word (code-instructions code-component) offs)))))
         ;; Loop over the embedded routines
         (let ((list name->addr)
               (obj-size (code-object-size code-component)))
@@ -1111,66 +1119,10 @@
                   (dumpwords (int-sap code-physaddr)
                              (code-header-words code) output header-exceptions code-addr)
                   (write-string (get-output-stream-string temp-output) output))))
-             ((functionp (%code-debug-info code))
-              (unless seen-trampolines
-                (setq seen-trampolines t)
-                (format output "lisp_trampolines:~%"))
-              (let* ((sap (int-sap (translate-ptr code-addr spaces)))
-                     (tramp-fun (sap-ref-word sap (ash 2 word-shift))))
-                (aver (not (in-bounds-p tramp-fun code-bounds)))
-                (format output " .quad ~{0x~x~^,~}~%"
-                        (loop for i from 0 by n-word-bytes repeat 6
-                              collect (sap-ref-word sap i)))))
              (t
               (error "Strange code component: ~S" code)))
            (incf code-addr objsize)))
-        (#.fdefn-widetag
-         (unless seen-fdefns
-           (format output "~%# FDEFNs~%")
-           (setq seen-fdefns t))
-         (let* ((ptr (translate-ptr code-addr spaces))
-                (fdefn (%make-lisp-obj (logior ptr other-pointer-lowtag)))
-                (name (fun-name-from-core (fdefn-name fdefn) core))
-                (c-name (c-name name core pp-state "F")))
-           (format output "~a: # ~x~% .size ~0@*~a, 32~%"
-                     (c-symbol-quote c-name)
-                     (logior code-addr other-pointer-lowtag))
-           (flet ((relativize (slot &aux (x (sap-ref-word (int-sap ptr) (ash slot word-shift))))
-                    (if (in-bounds-p x code-bounds)
-                        (format nil "CS+0x~x" (- x (bounds-low code-bounds)))
-                        (format nil "0x~x" x))))
-             (format output " .quad 0x~x, 0x~x, ~a, ~a~%"
-                     (sap-ref-word (int-sap ptr) 0)
-                     (sap-ref-word (int-sap ptr) 8)
-                     (relativize fdefn-fun-slot)
-                     (relativize fdefn-raw-addr-slot)))
-           (incf code-addr (* 4 n-word-bytes))))
-        (#.funcallable-instance-widetag
-         (unless seen-gfs
-           (setq seen-gfs t)
-           (when seen-trampolines
-             (format output " .size lisp_trampolines, .-lisp_trampolines~%")))
-         (let* ((sap (int-sap (translate-ptr code-addr spaces)))
-                (fin-fun (sap-ref-word sap (ash 2 word-shift)))
-                (code-space-p (in-bounds-p fin-fun code-bounds))
-                (slots (translate (sap-ref-lispobj sap (ash 3 word-shift)) spaces))
-                (name (and (> (length (the simple-vector slots)) +gf-name-slot+)
-                           (svref slots +gf-name-slot+)))
-                (c-name
-                 (c-name
-                  (if (or (not name)
-                          (eql (get-lisp-obj-address name) unbound-marker-widetag))
-                      "unnamed"
-                      (fun-name-from-core name core))
-                  core pp-state "G")))
-           (format output "~a:~% .size ~0@*~a, 48~%" (c-symbol-quote c-name))
-           (format output " .quad 0x~x, .+24, ~:[~;CS+~]0x~x~{, 0x~x~}~%"
-                   (sap-ref-word sap 0)
-                   code-space-p
-                   (if code-space-p (- fin-fun (bounds-low code-bounds)) fin-fun)
-                   (loop for i from (ash 3 word-shift) by n-word-bytes repeat 3
-                         collect (sap-ref-word sap i))))
-         (incf code-addr (* 6 n-word-bytes))))))
+        )))
 
   ;; coreparse uses the 'lisp_jit_code' symbol to set varyobj_free_pointer
   ;; The intent is that compilation to memory can use this reserved area
@@ -1253,7 +1205,7 @@
     (addend (signed 64))))
 
 (defun make-elf64-sym (name info)
-  (let ((a (make-array 24 :element-type '(unsigned-byte 8))))
+  (let ((a (make-array 24 :element-type '(unsigned-byte 8) :initial-element 0)))
     (with-pinned-objects (a)
       (setf (sap-ref-32 (vector-sap a) 0) name
             (sap-ref-8 (vector-sap a) 4) info))
@@ -1527,15 +1479,18 @@
              (return-from scan-obj))
            (case widetag
              (#.instance-widetag
-              (let ((layout (truly-the layout (translate (%instance-layout obj) spaces))))
-                (do-instance-tagged-slot (i obj :layout layout)
-                  (scanptr vaddr obj (1+ i))))
+              (let ((type (truly-the layout (translate (%instance-layout obj) spaces))))
+                (do-layout-bitmap (i taggedp type (%instance-length obj))
+                  (when taggedp
+                    (scanptr vaddr obj (1+ i)))))
               (return-from scan-obj))
              (#.simple-vector-widetag
               (let ((len (length (the simple-vector obj))))
-                (when (eql (logand (get-header-data obj) #xFF) vector-addr-hashing-subtype)
+                (when (logtest (get-header-data obj) vector-addr-hashing-flag)
                   (do ((i 2 (+ i 2)) (needs-rehash))
-                      ((= i len)
+                      ;; Refer to the figure at the top of src/code/hash-table.lisp.
+                      ;; LEN is an odd number.
+                      ((>= i (1- len))
                        (when needs-rehash
                          (setf (svref obj 1) 1)))
                     ;; A weak or EQ-based hash table any of whose keys is a function
@@ -1560,10 +1515,10 @@
                              (+ core-offs n-word-bytes)
                              word)))
               (when (eq widetag funcallable-instance-widetag)
-                (let* ((layout (truly-the layout (translate (%fun-layout obj) spaces)))
+                (let* ((layout (truly-the sb-vm:layout (translate (%fun-layout obj) spaces)))
                        (bitmap (%raw-instance-ref/signed-word
-                                layout (sb-kernel::type-dd-length sb-kernel:layout))))
-                  (unless (= (sb-kernel:layout-bitmap-words layout) 1)
+                                layout (sb-kernel::type-dd-length sb-vm:layout))))
+                  (unless (= (sb-kernel:bitmap-nwords layout) 1)
                     (error "Strange funcallable-instance bitmap"))
                   (unless (eql bitmap sb-kernel:+layout-all-tagged+)
                       ;; tagged slots precede untagged slots,
@@ -1666,12 +1621,12 @@
                              (alien-funcall
                               (extern-alien "load_core_bytes"
                                             (function system-area-pointer
-                                                      int int unsigned unsigned))
+                                                      int int unsigned unsigned int))
                               (sb-sys:fd-stream-fd ,stream)
-                              ;; Skip the core header
-                              (+ ,start +backend-page-bytes+)
+                              (+ ,start +backend-page-bytes+) ; Skip the core header
                               0 ; place it anywhere
-                              (* ,npages +backend-page-bytes+)))
+                              (* ,npages +backend-page-bytes+) ; len
+                              0))
                        ,@body)
                   (when ,sap-var
                     (alien-funcall
@@ -1748,14 +1703,15 @@
                       ;; adjust this entry's start page in the new core
                       (decf data-page page-adjust)))))
             (#.page-table-core-entry-type-code
-             (aver (= len 3))
-             (symbol-macrolet ((nbytes (%vector-raw-bits core-header (1+ ptr)))
-                               (data-page (%vector-raw-bits core-header (+ ptr 2))))
+             (aver (= len 4))
+             (symbol-macrolet ((n-ptes (%vector-raw-bits core-header (+ ptr 1)))
+                               (nbytes (%vector-raw-bits core-header (+ ptr 2)))
+                               (data-page (%vector-raw-bits core-header (+ ptr 3))))
                (aver (= data-page original-total-npages))
                (aver (= (ceiling (space-nwords
                                   (find dynamic-core-space-id spaces :key #'space-id))
-                                 (/ +backend-page-bytes+ n-word-bytes))
-                        (%vector-raw-bits core-header ptr))) ; number of PTEs
+                                 (/ sb-vm:gencgc-page-bytes n-word-bytes))
+                        n-ptes))
                (when verbose
                  (format t "PTE: page=~5x~40tbytes=~8x~%" data-page nbytes))
                (push (cons data-page nbytes) copy-actions)
@@ -1837,8 +1793,8 @@
                             ("*COMPILE-TO-MEMORY-SPACE*" . "DYNAMIC")))
               (destructuring-bind (symbol . value) item
                 (%set-symbol-global-value
-                 (find-target-symbol "SB-C" symbol map)
-                 (find-target-symbol "KEYWORD" value map :logical))))
+                 (find-target-symbol (package-id "SB-C") symbol map)
+                 (find-target-symbol (package-id "KEYWORD") value map :logical))))
             ;;
             (dolist (space data-spaces) ; Copy pages from memory
               (let ((start (space-physaddr space map))
@@ -1897,9 +1853,9 @@
                (when (plusp nwords)
                  (push (make-space id addr data-page 0 nwords) spaces))))
             (#.page-table-core-entry-type-code
-             (aver (= len 3))
-             (symbol-macrolet ((nbytes (%vector-raw-bits core-header (1+ ptr)))
-                               (data-page (%vector-raw-bits core-header (+ ptr 2))))
+             (aver (= len 4))
+             (symbol-macrolet ((nbytes (%vector-raw-bits core-header (+ ptr 2)))
+                               (data-page (%vector-raw-bits core-header (+ ptr 3))))
                (aver (= data-page total-npages))
                (setq core-size (+ (* total-npages +backend-page-bytes+) nbytes))))))
         (incf core-size +backend-page-bytes+) ; add in core header page
